/**
 * 
 */
package de.msponer.android.yac.activities.images;

import java.io.IOException;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.PixelFormat;
import android.hardware.Camera;
import android.os.Bundle;
import android.util.Log;
import android.view.Display;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.view.WindowManager;
import de.msponer.android.yac.R;

/**
 * Activity for taking pictures using the camera. Complete screen shows picture
 * preview. On click/tap the picture will be taken and returned to the previous
 * activity.
 * 
 * @author msponer
 */
public class CameraActivity extends Activity implements OnClickListener, SurfaceHolder.Callback {

	/**
	 * The activity tag.
	 */
	private static final String TAG = "CameraTest";

	/**
	 * The picture compression level.
	 */
	private static final int COMPRESSION_LEVEL = 50;
	
	/**
	 * The rotation degree.
	 */
	private static final int ROTATION_DEGREE = 90;
	
	/**
	 * The camera.
	 */
	private Camera mCamera;

	/**
	 * Status flag indicating a running preview.
	 */
	private boolean mPreviewRunning = false;

	/**
	 * The view id.
	 */
	private int mViewId;

	/**
	 * The surfcae view.
	 */
	private SurfaceView mSurfaceView;
	
	/**
	 * The surfcae holder.
	 */
	private SurfaceHolder mSurfaceHolder;
	
	/**
	 * {@inheritDoc} Initializes the activity.
	 */
	@Override
	public void onCreate(Bundle icicle) {
		super.onCreate(icicle);

		Log.d(TAG, "onCreate");

		Bundle extras = getIntent().getExtras();
		mViewId = extras.getInt("VIEW_ID");

		getWindow().setFormat(PixelFormat.JPEG);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
				WindowManager.LayoutParams.FLAG_FULLSCREEN);
		setContentView(R.layout.camera);
		mSurfaceView = (SurfaceView) findViewById(R.id.surface_camera);
		mSurfaceView.setOnClickListener(this);
		mSurfaceHolder = mSurfaceView.getHolder();
		mSurfaceHolder.addCallback(this);
		mSurfaceHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
	}

	@Override
	protected void onRestoreInstanceState(Bundle savedInstanceState) {
		super.onRestoreInstanceState(savedInstanceState);
	}

	/**
	 * The callback to return the taken picture.
	 */
	private Camera.PictureCallback mPictureCallback = new Camera.PictureCallback() {
		public void onPictureTaken(byte[] imageData, Camera c) {

			Log.d(TAG, "pictureCallback");

			if (imageData != null) {

				Intent mIntent = new Intent();

				String base64Binary = getImageBase64String(imageData, COMPRESSION_LEVEL);
				mCamera.startPreview();
				Log.d(TAG, base64Binary);
				mIntent.putExtra("VIEW_ID", mViewId);
				mIntent.putExtra("BASE64_BINARY", base64Binary);
				setResult(RESULT_OK, mIntent);

				finish();
			}
		}
	};

	@Override
	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
	}

	/**
	 * {@inheritDoc}
	 * Opens the camera preview.
	 */
	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		Log.d(TAG, "surfaceCreated");
		mCamera = Camera.open();
	}

	/**
	 * {@inheritDoc}
	 * Switch the preview orientation on surface changed.
	 */
	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {
		Log.d(TAG, "surfaceChanged");

		if (mPreviewRunning) {
			mCamera.stopPreview();
		}

		Camera.Parameters p = mCamera.getParameters();
		p.setPreviewSize(w, h);
		
		if (w < h) {
			p.set("orientation", "portrait");
			p.set("rotation", ROTATION_DEGREE);
			p.setPreviewSize(h, w);
		}

		mCamera.setParameters(p);
		try {
			mCamera.setPreviewDisplay(holder);
		} catch (IOException e) {
			e.printStackTrace();
		}
		mCamera.startPreview();
		mPreviewRunning = true;
	}

	/**
	 * {@inheritDoc}
	 * Stops picture preview and releases the camera.
	 */
	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		Log.d(TAG, "surfaceDestroyed");
		mCamera.stopPreview();
		mPreviewRunning = false;
		mCamera.release();
	}

	/**
	 * {@inheritDoc}
	 * Takes a picture on click.
	 */
	@Override
	public void onClick(View arg0) {
		mCamera.takePicture(null, mPictureCallback, mPictureCallback);
	}

	/**
	 * Decodes the bitmap and returns the resulting picture as BASE64 encoded string.
	 * @param imageData The image data.
	 * @param quality The quality level (compression level).
	 * @return The camera picture as BASE64 encoded string.
	 */
	public String getImageBase64String(byte[] imageData, int quality) {

		try {

			BitmapFactory.Options options = new BitmapFactory.Options();
			options.inSampleSize = 1;

			Bitmap myImage = BitmapFactory.decodeByteArray(imageData, 0, imageData.length, options);

			Display display = getWindowManager().getDefaultDisplay();
			int width = display.getWidth();
			int height = display.getHeight();

			return new BitmapResizer().getBase64EncodedResizedBitmap(myImage, width, height);

		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

}
