/**
 * 
 */
package de.msponer.android.yac.activities.images;

import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;

/**
 * View component for painting signatures using the touch screen.
 * 
 * @author msponer
 */
public class SigningView extends View {

	/**
	 * The stroke width.
	 */
	private static final int STROKE_WIDTH = 3;
	
	/**
	 * The canvas to paint on.
	 */
	private Canvas mCanvas;

	/**
	 * The path to paint.
	 */
	private Path mPath;

	/**
	 * The bitmap that contains the paining.
	 */
	private Bitmap mBitmap;

	/**
	 * The painting.
	 */
	private Paint mPaint;

	/**
	 * The signature as list of paths.
	 */
	private List<Path> mSignature = new ArrayList<Path>();

	/**
	 * Constructor that initializes the view.
	 * 
	 * @param context
	 *            The activity context
	 */
	public SigningView(Context context) {
		super(context);
		initView();
	}

	/**
	 * Constructor that initializes the view.
	 * 
	 * @param context
	 *            The activity context
	 * @param attrs
	 *            The set of attributes
	 */
	public SigningView(Context context, AttributeSet attrs) {
		super(context, attrs);
		initView();
	}

	/**
	 * Constructor that initializes the view.
	 * 
	 * @param context
	 *            The activity context
	 * @param attrs
	 *            The set of attributes
	 * @param params
	 *            The params count.
	 */
	public SigningView(Context context, AttributeSet attrs, int params) {
		super(context, attrs, params);
		initView();
	}

	/**
	 * Initializes the view. Configuring the current paint object.
	 */
	private void initView() {
		mPaint = new Paint();
		mPaint.setDither(true);
		mPaint.setColor(Color.BLACK);
		mPaint.setStyle(Paint.Style.STROKE);
		mPaint.setStrokeJoin(Paint.Join.ROUND);
		mPaint.setStrokeCap(Paint.Cap.ROUND);
		mPaint.setStrokeWidth(STROKE_WIDTH);
	}

	/**
	 * {@inheritDoc} Creates a new canvas and bitmap on size changed.
	 */
	@Override
	protected void onSizeChanged(int w, int h, int oldw, int oldh) {

		Log.d("SIGNING", "onSizeChanged");

		int curW = 0;
		if (mBitmap != null) {
			curW = mBitmap.getWidth();
		}

		int curH = 0;
		if (mBitmap != null) {
			curH = mBitmap.getHeight();
		}

		if (curW >= w && curH >= h) {
			return;
		}

		if (curW < w) {
			curW = w;
		}

		if (curH < h) {
			curH = h;
		}

		mBitmap = Bitmap.createBitmap(curW, curH, Bitmap.Config.RGB_565);
		mCanvas = new Canvas(mBitmap);
		mCanvas.drawColor(Color.WHITE);
	}

	/**
	 * {@inheritDoc} Process the motion events and invokes the path drawing.
	 */
	@Override
	public boolean onTouchEvent(MotionEvent event) {
		Log.d("SIGNING", "onTouchEvent");
		if (event.getAction() == MotionEvent.ACTION_DOWN) {
			mPath = new Path();
			mPath.moveTo(event.getX(), event.getY());
			mPath.lineTo(event.getX(), event.getY());
			mSignature.add(mPath);
		} else if (event.getAction() == MotionEvent.ACTION_MOVE) {
			mPath.lineTo(event.getX(), event.getY());
		} else if (event.getAction() == MotionEvent.ACTION_UP) {
			mPath.lineTo(event.getX(), event.getY());
		}
		drawPath();
		return true;
	}

	/**
	 * {@inheritDoc} Draws the bitmap onto the given canvas.
	 */
	@Override
	protected void onDraw(Canvas canvas) {
		Log.d("SIGNING", "onDraw");
		if (mBitmap != null) {
			canvas.drawBitmap(mBitmap, 0, 0, null);
		}
	}

	/**
	 * Draws the bitmap and invalidates the view.
	 */
	private void drawPath() {
		Log.d("SIGNING", "drawPath");
		for (Path path : mSignature) {
			mCanvas.drawPath(path, mPaint);
		}
		invalidate();
	}

	/**
	 * Returns the current bitmap.
	 * 
	 * @return The current bitmap.
	 */
	public Bitmap getBitmap() {
		return this.mBitmap;
	}
}
