/**
 * 
 */
package de.msponer.android.yac.activities.position;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.Point;
import android.graphics.RectF;
import android.location.Location;
import android.location.LocationManager;
import android.os.Bundle;
import android.util.Log;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.MyLocationOverlay;
import com.google.android.maps.Overlay;

import de.msponer.android.yac.R;

/**
 * Activity for localizing a work item on a MapView.
 * 
 * @author msponer
 * 
 */
public class WorkItemMapActivity extends MapActivity {

	/**
	 * The activity tag.
	 */
	private static final String TAG = "WorkItemMapActivity";

	/**
	 * The map view component.
	 */
	private MapView mMapView;

	/**
	 * The map controller.
	 */
	private MapController mMapController;

	/**
	 * The overlay for "my" location.
	 */
	private MyLocationOverlay mMyLocationOverlay;

	/**
	 * The work item location overlay.
	 */
	private WorkItemLocationOverlay mWorkItemLocationOverlay;

	/**
	 * The work item task name.
	 */
	private String mTaskname;

	@Override
	protected boolean isRouteDisplayed() {
		return false;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		initMapView();

		initMyLocationOverlay();

		this.mTaskname = getIntent().getExtras().getString("WORKITEM_TASKNAME");
		String latitude = getIntent().getExtras().getString("WORKITEM_LATITUDE");
		String longitude = getIntent().getExtras().getString("WORKITEM_LONGITUDE");

		if (latitude != null && latitude.length() > 0 && longitude != null
				&& longitude.length() > 0) {
			Double latitudeDouble = Double.parseDouble(latitude);
			Double longitudeDouble = Double.parseDouble(longitude);

			GeoPoint workItemGeoPoint = new GeoPoint((int) (latitudeDouble * 1E6),
					(int) (longitudeDouble * 1E6));

			initWorkItemLocationOverlay(this.mTaskname, workItemGeoPoint);
		}
	}

	@Override
	protected void onResume() {
		mMyLocationOverlay.enableMyLocation();
		mMyLocationOverlay.enableCompass();
		mMapView.invalidate();
		super.onResume();
	}

	@Override
	protected void onPause() {

		mMyLocationOverlay.disableMyLocation();
		mMyLocationOverlay.disableCompass();
		super.onPause();
	}

	/**
	 * Initializes the map view.
	 */
	private void initMapView() {
		setContentView(R.layout.mapview_workitem);

		mMapView = (MapView) findViewById(R.id.mapview);

		mMapController = mMapView.getController();

		mMapView.setBuiltInZoomControls(true);

		mMapView.setSatellite(true);
		// mapView.setStreetView(true);

	}

	/**
	 * Initializes "my" location overlay.
	 */
	private void initMyLocationOverlay() {
		mMyLocationOverlay = new MyLocationOverlay(this, this.mMapView) {
			public synchronized void onLocationChanged(Location newLocation) {
				super.onLocationChanged(newLocation);
				GeoPoint myPosition = new GeoPoint((int) (newLocation.getLatitude() * 1E6),
						(int) (newLocation.getLongitude() * 1E6));

				mMapController.animateTo(myPosition);
			};
		};

		mMapView.getOverlays().add(mMyLocationOverlay);
		mMyLocationOverlay.enableMyLocation();
		mMyLocationOverlay.enableCompass();
		mMyLocationOverlay.runOnFirstFix(new Runnable() {
			@Override
			public void run() {
				mMapController.animateTo(mMyLocationOverlay.getMyLocation());
			}
		});
	}

	/**
	 * Initializes the work item location overlay.
	 * 
	 * @param taskname
	 *            The task name to display.
	 * @param workItemGeoPoint
	 *            The geo point to place the work item at.
	 */
	private void initWorkItemLocationOverlay(String taskname, GeoPoint workItemGeoPoint) {

		this.mWorkItemLocationOverlay = new WorkItemLocationOverlay(taskname, workItemGeoPoint);
		this.mMapView.getOverlays().add(this.mWorkItemLocationOverlay);

		this.mMapView.postInvalidate();
	}

	/**
	 * Work item location overlay class.
	 * 
	 * @author marcel
	 * 
	 */
	public class WorkItemLocationOverlay extends Overlay {

		/**
		 * The task name to display.
		 */
		private String mTaskname;

		/**
		 * The geo point of the work item.
		 */
		private GeoPoint mWorkItemGeoPoint;

		/**
		 * The work item location point.
		 */
		private final Point mWorkItemLocationPoint = new Point();

		/**
		 * "My" location point.
		 */
		private final Point mMyLocationPoint = new Point();

		/**
		 * A rectangle.
		 */
		private final RectF mRect = new RectF();

		/**
		 * A paint object.
		 */
		private final Paint mPaint = new Paint();

		/**
		 * Constructor setting taskname and geo point.
		 * 
		 * @param taskname
		 *            The taskname to display.
		 * @param workItemGeoPoint
		 *            The geopoint of the work item.
		 */
		public WorkItemLocationOverlay(final String taskname, GeoPoint workItemGeoPoint) {
			this.mTaskname = taskname;
			this.mWorkItemGeoPoint = workItemGeoPoint;
		}

		@Override
		public void draw(Canvas canvas, MapView mapView, boolean shadow) {
			super.draw(canvas, mapView, shadow);

			GeoPoint myPosition = mMyLocationOverlay.getMyLocation();

			if (myPosition == null) {
				Log.d(TAG, "myPosition == NULL");
				LocationManager locationManager = (LocationManager) 
					getSystemService(Context.LOCATION_SERVICE);
				Location lastKnownLocation = locationManager
						.getLastKnownLocation(LocationManager.GPS_PROVIDER);
				if (lastKnownLocation != null) {
					Log.d(TAG, "Last-Known-Location: " + lastKnownLocation.getLatitude() + ", "
							+ lastKnownLocation.getLongitude());
					myPosition = new GeoPoint((int) (lastKnownLocation.getLatitude()),
							(int) (lastKnownLocation.getLongitude()));
				}
			}

			if (myPosition != null) {

				mapView.getProjection().toPixels(mWorkItemGeoPoint, mWorkItemLocationPoint);

				mapView.getProjection().toPixels(myPosition, mMyLocationPoint);

				float textWidth = mPaint.measureText("My Position");
				float textHeight = mPaint.getTextSize();
				mRect.set(mMyLocationPoint.x + 13, mMyLocationPoint.y - textHeight,
						mMyLocationPoint.x + 13 + 8 + textWidth, mMyLocationPoint.y + 4);
				mPaint.setARGB(128, 255, 255, 255);
				mPaint.setStyle(Style.FILL);
				canvas.drawRect(mRect, mPaint);

				mPaint.setARGB(255, 0, 0, 0);
				canvas.drawText("You", mMyLocationPoint.x + 13 + 4, mMyLocationPoint.y, mPaint);

				// Draw line: My current position -> workitemlocation
				canvas.drawLine(mMyLocationPoint.x, mMyLocationPoint.y, mWorkItemLocationPoint.x,
						mWorkItemLocationPoint.y, mPaint);

				// set drawing area
				mRect.set(mWorkItemLocationPoint.x - 5, mWorkItemLocationPoint.y + 5,
						mWorkItemLocationPoint.x + 5, mWorkItemLocationPoint.y - 5);

				// Red oval for my own location
				mPaint.setARGB(255, 200, 0, 30);
				mPaint.setStyle(Style.FILL);
				canvas.drawOval(mRect, mPaint);

				// black circle around the oval
				mPaint.setARGB(255, 0, 0, 0);
				mPaint.setStyle(Style.STROKE);
				canvas.drawCircle(mWorkItemLocationPoint.x, mWorkItemLocationPoint.y, 5, mPaint);

				textWidth = mPaint.measureText(mTaskname);
				textHeight = mPaint.getTextSize();
				mRect.set(mWorkItemLocationPoint.x + 8, mWorkItemLocationPoint.y - textHeight,
						mWorkItemLocationPoint.x + 8 + 8 + textWidth, mWorkItemLocationPoint.y + 4);
				mPaint.setARGB(128, 64, 64, 64);
				mPaint.setStyle(Style.FILL);
				canvas.drawRect(mRect, mPaint);
				mPaint.setARGB(255, 255, 255, 255);
				canvas.drawText(mTaskname, mWorkItemLocationPoint.x + 8 + 4,
						mWorkItemLocationPoint.y, mPaint);
			}
		}
	}
}
