/**
 * 
 */
package de.msponer.android.yac.activities.position;

import java.util.List;

import android.content.SharedPreferences;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.os.Bundle;
import android.util.Log;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.MyLocationOverlay;
import com.google.android.maps.Overlay;

import de.msponer.android.yac.R;
import de.msponer.android.yac.db.QueryHelper;
import de.msponer.android.yac.model.MobileWorkItem;

/**
 * Activity to display all localized work items on a MapView.
 * 
 * @author msponer
 * 
 */
public class WorkQueueMapActivity extends MapActivity {

	/**
	 * The activity tag.
	 */
	private static final String TAG = "WorkQueueMapActivity";

	/**
	 * The map view.
	 */
	private MapView mapView;

	/**
	 * The map controller.
	 */
	private MapController mapController;

	/**
	 * "My" location overlay.
	 */
	private MyLocationOverlay myLocationOverlay;

	/**
	 * List of all localized mobile work items.
	 */
	private List<MobileWorkItem> mLocalizedMobileWorkItems;

	@Override
	protected boolean isRouteDisplayed() {
		return false;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		initMapView();

		initMyLocationOverlay();

	}

	@Override
	protected void onResume() {

		SharedPreferences sharedPreferences = getSharedPreferences(
				"de.msponer.android.yac_preferences", MODE_PRIVATE);

		String lonKey = sharedPreferences.getString("prefs_longitude_key", "Longitude");
		String latKey = sharedPreferences.getString("prefs_latitute_key", "Latitude");

		super.onResume();

		Log.d(TAG, "onResume ...");

		mapView.invalidate();

		try {
			this.mLocalizedMobileWorkItems = new QueryHelper(this).queryForAllLocalizedWorkitems(
					lonKey, latKey);
		} catch (Exception e) {
			Log.e(TAG, "Error during DB-Query.");
		}

		if (this.mLocalizedMobileWorkItems != null && this.mLocalizedMobileWorkItems.size() > 0) {
			initWorkItemLocations();
		}

		myLocationOverlay.enableMyLocation();
		myLocationOverlay.enableCompass();

	}

	@Override
	protected void onPause() {

		Log.d(TAG, "onPause ...");
		myLocationOverlay.disableMyLocation();
		myLocationOverlay.disableCompass();
		super.onPause();
	}

	/**
	 * Initializes the map view.
	 */
	private void initMapView() {
		setContentView(R.layout.mapview_workitem);

		mapView = (MapView) findViewById(R.id.mapview);

		mapController = mapView.getController();

		mapView.setBuiltInZoomControls(true);

		mapView.setSatellite(false);
		mapView.setStreetView(true);

	}

	/**
	 * Initializes "my" location over view.
	 */
	private void initMyLocationOverlay() {
		myLocationOverlay = new MyLocationOverlay(this, this.mapView) {
			public synchronized void onLocationChanged(Location newLocation) {
				super.onLocationChanged(newLocation);
				GeoPoint myPosition = new GeoPoint((int) (newLocation.getLatitude() * 1E6),
						(int) (newLocation.getLongitude() * 1E6));

				mapController.animateTo(myPosition);
			};
		};

		mapView.getOverlays().add(myLocationOverlay);
		myLocationOverlay.enableMyLocation();
		myLocationOverlay.enableCompass();
		myLocationOverlay.runOnFirstFix(new Runnable() {
			@Override
			public void run() {
				mapController.animateTo(myLocationOverlay.getMyLocation());
			}
		});
	}

	/**
	 * Initializes work item locations using a TappableWorkListOverlay.
	 */
	private void initWorkItemLocations() {
		List<Overlay> mapOverlays = mapView.getOverlays();

		Drawable icon = getResources().getDrawable(R.drawable.pin);
		// icon.setBounds(0, 0, 0 + icon.getIntrinsicWidth()/4, 0 +
		// icon.getIntrinsicHeight()/4);

		TappableWorkListOverlay<MobileWorkItemOverlay> worklistOverlay = 
			new TappableWorkListOverlay<MobileWorkItemOverlay>(icon, mapView.getContext());

		for (MobileWorkItem mwi : this.mLocalizedMobileWorkItems) {

			if (mwi.getLatitude() != null && mwi.getLatitude().length() > 0
					&& mwi.getLongitude() != null && mwi.getLongitude().length() > 0) {

				Double longitudeDouble = Double.parseDouble(mwi.getLongitude());
				Double latitudeDouble = Double.parseDouble(mwi.getLatitude());

				int longitude = (int) (longitudeDouble * 1E6);
				int latitude = (int) (latitudeDouble * 1E6);

				GeoPoint point = new GeoPoint(latitude, longitude);

				MobileWorkItemOverlay overlayitem = new MobileWorkItemOverlay(point, mwi);

				worklistOverlay.addOverlay(overlayitem);
			}
		}

		Log.d(TAG, "Overlay-Size: " + worklistOverlay.size());

		if (mapOverlays.size() == 2) {
			Log.d(TAG, "Overwriting mapoverlay");
			mapOverlays.set(1, worklistOverlay);
		} else {
			mapOverlays.add(worklistOverlay);
		}

		Log.d(TAG, "MapOverlays: " + mapOverlays.size());

		mapView.invalidate();

	}

}
