package de.msponer.android.yac.activities.workqueues;

import java.sql.SQLException;

import org.jdom.Element;
import org.yawlfoundation.yawl.util.JDOMUtil;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.ComponentName;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ExpandableListAdapter;
import android.widget.ExpandableListView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.j256.ormlite.android.apptools.OrmLiteBaseActivity;
import com.j256.ormlite.dao.Dao;

import de.msponer.android.yac.R;
import de.msponer.android.yac.adapter.WorkItemDataExpendableListAdapter;
import de.msponer.android.yac.db.DatabaseHelper;
import de.msponer.android.yac.model.MobileWorkItem;
import de.msponer.android.yac.model.State;
import de.msponer.android.yac.services.impl.YConnectionService;
import de.msponer.android.yac.services.impl.YConnectionService.YConnectionServiceBinder;

/**
 * Activity for displaying work item details and manipulating the work item's
 * life cycle. Different Intent actions are support.
 * 
 * @author marcel
 * 
 */
public class WorkItemActivity extends OrmLiteBaseActivity<DatabaseHelper> {

	/**
	 * The activity tag.
	 */
	private static final String TAG = "WorkItemActivity";

	/**
	 * Intent action: Accept work item.
	 */
	public static final String ACCEPT_OFFER_INTENT = "de.msponer.yac.ACCEPT_OFFER";

	/**
	 * Intent action: Accept and start work item.
	 */
	public static final String ACCEPT_START_INTENT = "de.msponer.yac.ACCEPT_START";

	/**
	 * Intent action: Start work item.
	 */
	public static final String START_INTENT = "de.msponer.yac.START_WORKITEM";

	/**
	 * Intent action: Deallocate work item.
	 */
	public static final String DEALLOCATE_INTENT = "de.msponer.yac.DEALLOCATE_WORKITEM";

	/**
	 * Intent action: Skip work item.
	 */
	public static final String SKIP_INTENT = "de.msponer.yac.SKIP_WORKITEM";

	/**
	 * Intent action: View/edit work item.
	 */
	public static final String EDIT_INTENT = "de.msponer.yac.EDIT_WORKITEM";

	/**
	 * Intent action: Suspend work item.
	 */
	public static final String SUSPEND_INTENT = "de.msponer.yac.SUSPEND_WORKITEM";

	/**
	 * Intent action: Complete work item.
	 */
	public static final String COMPLETE_INTENT = "de.msponer.yac.COMPLETE_WORKITEM";

	/**
	 * Intent action: Unsuspend work item.
	 */
	public static final String UNSUSPEND_INTENT = "de.msponer.yac.UNSUSPEND_WORKITEM";

	/**
	 * Result code: Accept work item.
	 */
	public static final int ACCEPT_OFFER = 10;

	/**
	 * Result code: Accept and start work item.
	 */
	public static final int ACCEPT_AND_START = 11;

	/**
	 * Result code: Chain work item.
	 */
	public static final int CHAIN = 12;

	/**
	 * Result code: Start work item.
	 */
	public static final int START = 13;

	/**
	 * Result code: Deallocate work item.
	 */
	public static final int DEALLOCATE = 14;

	/**
	 * Result code: Delegate work item.
	 */
	public static final int DELEGATE = 15;

	/**
	 * Result code: Skip work item.
	 */
	public static final int SKIP = 16;

	/**
	 * Result code: Pile work item.
	 */
	public static final int PILE = 17;

	/**
	 * Result code: View/edit work item.
	 */
	public static final int VIEW_EDIT = 18;

	/**
	 * Result code: Suspend work item.
	 */
	public static final int SUSPEND = 19;

	/**
	 * Result code: Reallocate (sl) work item.
	 */
	public static final int REALLOCATE_SL = 20;

	/**
	 * Result code: Reallocate (sf) work item.
	 */
	public static final int REALLOCATE_SF = 21;

	/**
	 * Result code: New instance.
	 */
	public static final int NEW_INSTANCE = 22;

	/**
	 * Result code: Complete work item.
	 */
	public static final int COMPLETE = 23;

	/**
	 * Result code: Unsuspend work item.
	 */
	public static final int UNSUSPEND = 24;

	/**
	 * Result code: Update work item.
	 */
	public static final int UPDATE = 25;

	/**
	 * The current work item id.
	 */
	private int mCurrentWorkItemId;

	/**
	 * The current mobile work item.
	 */
	private MobileWorkItem mCurrentMobileWorkItem;

	/**
	 * The current work item XML data root.
	 */
	private Element mWorkItemXmlDataRoot;

	/**
	 * The mobile work item dao.
	 */
	private Dao<MobileWorkItem, Integer> mMobileWorkItemDao;

	/**
	 * The work item sync queue.
	 */
	private String mQueue;

	/**
	 * The activity request code.
	 */
	private int mRequestCode;

	/**
	 * Flag indicating the chosen sync mode.
	 */
	private boolean mLiveSyncEnabled;

	/**
	 * The YAWL connection intent.
	 */
	private Intent mYConnectionIntent;

	/**
	 * The intent action string.
	 */
	private String mIntentAction;

	/**
	 * The sync progress dialog.
	 */
	private ProgressDialog mDialog;

	/**
	 * The expandable data list view.
	 */
	private ExpandableListView mDataListView;

	/**
	 * The YAWL connection service.
	 */
	private YConnectionService mYConnectionService;

	/**
	 * The service connection.
	 */
	private ServiceConnection mYConnection = new ServiceConnection() {
		@Override
		public void onServiceConnected(ComponentName name, IBinder service) {
			mYConnectionService = ((YConnectionServiceBinder) service).getYConnectionService();
			doIntentAction();
		}

		@Override
		public void onServiceDisconnected(ComponentName name) {
			mYConnectionService = null;
		}
	};

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		Intent callingIntent = getIntent();
		mIntentAction = getIntent().getAction();
		mCurrentWorkItemId = callingIntent.getExtras().getInt("PRIMARY_KEY");

		try {
			mMobileWorkItemDao = getHelper().getDao(MobileWorkItem.class);
			if (mCurrentWorkItemId > 0) {
				mCurrentMobileWorkItem = mMobileWorkItemDao.queryForId(mCurrentWorkItemId);
			} else {
				@SuppressWarnings("unused")
				String caseId = callingIntent.getExtras().getString("CASE_ID");
			}

			mQueue = mCurrentMobileWorkItem.getResourceStatus();

			// Content-View is only needed, if intentaction is null
			if (mIntentAction == null) {
				setContentView(R.layout.work_item);

				// Set Text
				((TextView) findViewById(R.id.textViewWorkItemID)).setText(mCurrentMobileWorkItem
						.getID());
				((TextView) findViewById(R.id.textViewWorkItemTask)).setText(mCurrentMobileWorkItem
						.getTaskName());
				((TextView) findViewById(R.id.textViewWorkItemStatus))
						.setText(mCurrentMobileWorkItem.getStatus());
				((TextView) findViewById(R.id.textViewWorkItemCreated))
						.setText(mCurrentMobileWorkItem.getEnablementTimeString());

				createButtonView();

				createWorkitemDataList();
			} else {
				setContentView(new View(this));
			}

			// Setting Live-Sync Flag
			SharedPreferences sharedPreferences = getSharedPreferences(
					"de.msponer.android.yac_preferences", MODE_PRIVATE);
			this.mLiveSyncEnabled = sharedPreferences.getBoolean("prefs_livesync", true);

			mYConnectionIntent = new Intent(this, YConnectionService.class);

			if (mLiveSyncEnabled) {
				bindService(mYConnectionIntent, mYConnection, BIND_AUTO_CREATE);
				Log.d(TAG, "Binding Service: " + YConnectionService.class.getName());
			} else {
				doIntentAction();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Initializes the button view so that the user can invoke available work
	 * item actions to manipulate the life cycle.
	 */
	private void createButtonView() {

		LinearLayout buttonLayout = (LinearLayout) findViewById(R.id.linearLayoutWorkitemButtons);

		if (mQueue.equals("Offered")) {
			getLayoutInflater().inflate(R.layout.buttons_workitem_offered, buttonLayout);

			Button btnAccept = (Button) findViewById(R.id.buttonAcceptOffer);
			btnAccept.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					acceptOffer();
				}
			});

			Button btnAcceptAndStart = (Button) findViewById(R.id.buttonAcceptOfferAndStart);
			btnAcceptAndStart.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					acceptAndStartOffer();
				}
			});
		} else if (mQueue.equals("Allocated")) {
			getLayoutInflater().inflate(R.layout.buttons_workitem_allocated, buttonLayout);

			Button btnStart = (Button) findViewById(R.id.buttonStart);
			btnStart.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					startWorkitem();
				}
			});

			Button btnDeallocate = (Button) findViewById(R.id.buttonDeallocate);
			btnDeallocate.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					deallocate();
				}
			});

			Button btnSkip = (Button) findViewById(R.id.buttonSkip);
			btnSkip.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					skip();
				}
			});
		} else if (mQueue.equals("Started")) {
			getLayoutInflater().inflate(R.layout.buttons_workitem_started, buttonLayout);

			Button btnViewEdit = (Button) findViewById(R.id.buttonViewEdit);
			btnViewEdit.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					viewEdit();
				}
			});

			Button btnSuspend = (Button) findViewById(R.id.buttonSuspend);
			btnSuspend.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					suspend();
				}
			});

			Button btnComplete = (Button) findViewById(R.id.buttonComplete);
			btnComplete.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					complete();
				}
			});
		} else if (mQueue.equals("Suspended")) {
			getLayoutInflater().inflate(R.layout.buttons_workitem_suspended, buttonLayout);

			Button btnUnsuspend = (Button) findViewById(R.id.buttonUnsuspend);
			btnUnsuspend.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					unsuspend();
				}
			});
		}
	}

	/**
	 * Initializes the work item data list needed by the expandable list
	 * adapter.
	 */
	private void createWorkitemDataList() {

		this.mDataListView = (ExpandableListView) findViewById(R.id.listViewWorkItemData);

		String dataString;
		if (mCurrentMobileWorkItem.getUpdatedDataString() != null
				&& mCurrentMobileWorkItem.getUpdatedDataString().length() > 0) {
			dataString = mCurrentMobileWorkItem.getUpdatedDataString();
		} else {
			dataString = mCurrentMobileWorkItem.getDataListString();
		}

		try {
			this.mWorkItemXmlDataRoot = JDOMUtil.stringToElement(dataString);
		} catch (Exception e) {
			e.printStackTrace();
		}

		if (this.mWorkItemXmlDataRoot != null) {

			ExpandableListAdapter expListAdapter = new WorkItemDataExpendableListAdapter(this,
					this.mWorkItemXmlDataRoot);
			TextView noData = (TextView) findViewById(R.id.textViewNoWorkItemData);
			mDataListView.setEmptyView(noData);
			mDataListView.setAdapter(expListAdapter);
		}
	}

	@Override
	protected void onDestroy() {

		if (mLiveSyncEnabled) {
			unbindService(mYConnection);
			Log.d(TAG, "Unbinding Service: " + YConnectionService.class.getName());
		}

		super.onDestroy();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {

		if (mQueue.equals("Offered")) {
			getMenuInflater().inflate(R.menu.menu_workitem_offered, menu);
		} else if (mQueue.equals("Allocated")) {
			getMenuInflater().inflate(R.menu.menu_workitem_allocated, menu);
		} else if (mQueue.equals("Started")) {
			getMenuInflater().inflate(R.menu.menu_workitem_started, menu);
		} else if (mQueue.equals("Suspended")) {
			getMenuInflater().inflate(R.menu.menu_workitem_suspended, menu);
		}

		return super.onCreateOptionsMenu(menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menuWorkitemOfferedAccept:
			acceptOffer();
			break;
		case R.id.menuWorkitemOfferedAcceptAndStart:
			acceptAndStartOffer();
			break;
		case R.id.menuWorkitemOfferedChain:
			chainOffer();
			break;
		case R.id.menuWorkitemAllocatedStart:
			startWorkitem();
			break;
		case R.id.menuWorkitemAllocatedDeallocate:
			deallocate();
			break;
		case R.id.menuWorkitemAllocatedDelegate:
			delegate();
			break;
		case R.id.menuWorkitemAllocatedSkip:
			skip();
			break;
		case R.id.menuWorkitemAllocatedPile:
			pile();
			break;
		case R.id.menuWorkitemStartedViewEdit:
			viewEdit();
			break;
		case R.id.menuWorkitemStartedSuspend:
			suspend();
			break;
		case R.id.menuWorkitemStartedReallocateSL:
			reallocateSL();
			break;
		case R.id.menuWorkitemStartedReallocateSF:
			reallocateSF();
			break;
		case R.id.menuWorkitemStartedNewInstance:
			newInstance();
			break;
		case R.id.menuWorkitemStartedComplete:
			complete();
			break;
		case R.id.menuWorkitemSuspendedUnsuspend:
			unsuspend();
			break;
		default:
		}

		return super.onOptionsItemSelected(item);
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		Log.d(TAG, "ActivityResult: " + resultCode);

		switch (resultCode) {
		case Activity.RESULT_CANCELED:

			if (mIntentAction != null
			// || intentAction.length() > 0
			) {
				Log.d(TAG, "FINISH WITH RESULT 0");
				finishAndReturnResult(Activity.RESULT_CANCELED);
			}
			break;
		case EditWorkItemActivity.COMPLETED:
			complete();
			break;
		case EditWorkItemActivity.SAVED:
			update();
			break;
		default:
			break;
		}
	}

	/**
	 * Invokes intent actions.
	 */
	private void doIntentAction() {

		if (mIntentAction != null && mIntentAction.length() > 0) {
			if (mIntentAction.equals(ACCEPT_OFFER_INTENT)) {
				acceptOffer();
			} else if (mIntentAction.equals(ACCEPT_START_INTENT)) {
				acceptAndStartOffer();
			} else if (mIntentAction.equals(START_INTENT)) {
				startWorkitem();
			} else if (mIntentAction.equals(DEALLOCATE_INTENT)) {
				deallocate();
			} else if (mIntentAction.equals(SKIP_INTENT)) {
				skip();
			} else if (mIntentAction.equals(EDIT_INTENT)) {
				viewEdit();
			} else if (mIntentAction.equals(SUSPEND_INTENT)) {
				suspend();
			} else if (mIntentAction.equals(COMPLETE_INTENT)) {
				complete();
			} else if (mIntentAction.equals(UNSUSPEND_INTENT)) {
				unsuspend();
			}
		}
	}

	/**
	 * Accepts a work item and finishes the activity.
	 */
	private void acceptOffer() {

		doAction(MobileWorkItem.STATUS_RESOURCE_ALLOCATED, null, "OfferAccepted");

		finishAndReturnResult(WorkItemActivity.ACCEPT_OFFER);
	}

	/**
	 * Accepts and starts a work item and finishes the activity.
	 */
	private void acceptAndStartOffer() {

		doAction(MobileWorkItem.STATUS_RESOURCE_STARTED, MobileWorkItem.STATUS_EXECUTING,
				"OfferAccepted,Started");

		finishAndReturnResult(WorkItemActivity.ACCEPT_AND_START);
	}

	/**
	 * Chains a work item and finishes the activity.
	 */
	private void chainOffer() {
		/*
		 * Chain bedeutet OfferAccept&start und weise mir alle folgenden
		 * Workitem direkt zu und starte sie
		 * 
		 * Interface-Methode für chain nicht vorhanden!
		 */

		// try {
		// selectedItem.setResourceStatus(MobileWorkItem.statusResourceStarted);
		// selectedItem.setStatus(MobileWorkItem.statusExecuting);
		// selectedItem.setStatus(MobileWorkItem.statusExecuting);
		// Log.d(TAG, "WorkItem-Offer chained: " + selectedItem.getID());
		// selectedItem.setState(State.MODIFIED);
		// selectedItem.appendSyncAction("Chained");
		//
		// mobileWorkItemDao.update(selectedItem);
		//
		// removeSelectedWorkItem();
		//
		// } catch (SQLException e) {
		// e.printStackTrace();
		// }

		finishAndReturnResult(WorkItemActivity.CHAIN);
	}

	/**
	 * Starts a work item and finishes the activity.
	 */
	private void startWorkitem() {

		doAction(MobileWorkItem.STATUS_RESOURCE_STARTED, MobileWorkItem.STATUS_EXECUTING, 
				"Started");

		finishAndReturnResult(WorkItemActivity.START);
	}

	/**
	 * Deallocates a work item and finishes the activity.
	 */
	private void deallocate() {

		doAction(MobileWorkItem.STATUS_RESOURCE_UNOFFERED, null, "Deallocated");

		finishAndReturnResult(WorkItemActivity.DEALLOCATE);
	}

	/**
	 * Delegates a work item and finishes the activity.
	 */
	private void delegate() {
		// TODO Auto-generated method stub
		finishAndReturnResult(WorkItemActivity.DELEGATE);
	}

	/**
	 * Skips a work item and finishes the activity.
	 */
	private void skip() {

		doAction(MobileWorkItem.STATUS_RESOURCE_STARTED, MobileWorkItem.STATUS_COMPLETE, "Skipped");

		finishAndReturnResult(WorkItemActivity.SKIP);
	}

	/**
	 * Piles a work item and finishes the activity.
	 */
	private void pile() {

		doAction(MobileWorkItem.STATUS_RESOURCE_STARTED, MobileWorkItem.STATUS_EXECUTING, "Piled");

		finishAndReturnResult(WorkItemActivity.PILE);
	}

	/**
	 * Starts a work item if not yet started and invokes EditWorkItemActivity.
	 */
	private void viewEdit() {

		if (!mCurrentMobileWorkItem.getStatus().equals("Executing")) {
			// Start it,
			doAction(MobileWorkItem.STATUS_RESOURCE_STARTED, MobileWorkItem.STATUS_EXECUTING,
					"Started");
			Log.d(TAG, "Starting the WI...");
		}
		
		// update it,
		if (mLiveSyncEnabled) {
			
			mDialog = 
				ProgressDialog.show(this, "Please wait..", "Live-Sync in action", true, false);
			
			if (mYConnectionService != null) {
				int newId = mYConnectionService.liveSyncWorkitem(mCurrentMobileWorkItem);
				
				try {
					MobileWorkItem newMobileWorkItem = mMobileWorkItemDao.queryForId(newId);
					if (newMobileWorkItem != null) {
						this.mCurrentMobileWorkItem = newMobileWorkItem;
						this.mCurrentWorkItemId = newMobileWorkItem.getPrimaryKey();
						Log.d(TAG, "New-MOBWIR-Data:");
						Log.d(TAG, newMobileWorkItem.getDataListString());
						if (newMobileWorkItem.getUpdatedDataString() != null) {
							Log.d(TAG, "New-MOBWIR-UpdatedData:");
							Log.d(TAG, newMobileWorkItem.getUpdatedDataString());
						}						
					} 
				} catch (SQLException e) {
					e.printStackTrace();
				}
				
			}
			mDialog.dismiss();
			setProgressBarIndeterminateVisibility(false);
			
		}

		// and edit it ...
		Intent viewEditWiIntent = new Intent(this, EditWorkItemActivity.class);
		viewEditWiIntent.putExtra("PRIMARY_KEY", mCurrentWorkItemId);
		startActivityForResult(viewEditWiIntent, mRequestCode);
	}

	/**
	 * Suspends a work item and finishes the activity.
	 */
	private void suspend() {

		doAction(MobileWorkItem.STATUS_RESOURCE_SUSPENDED, null, "Suspended");

		finishAndReturnResult(WorkItemActivity.SUSPEND);
	}

	/**
	 * Reallocates (sl) a work item and finishes the activity.
	 */
	private void reallocateSL() {
		finishAndReturnResult(WorkItemActivity.REALLOCATE_SL);
	}

	/**
	 * Reallocates (sf) a work item and finishes the activity.
	 */
	private void reallocateSF() {
		finishAndReturnResult(WorkItemActivity.REALLOCATE_SF);
	}

	/**
	 * Creates a new work item instance and finishes the activity.
	 */
	private void newInstance() {
		// TODO: Create new wi instance
		finishAndReturnResult(WorkItemActivity.NEW_INSTANCE);
	}

	/**
	 * Updates a work item and finishes the activity.
	 */
	private void update() {

		Log.d(TAG, "Refreshing: " + mCurrentMobileWorkItem.getID());

		try {
			this.mMobileWorkItemDao.refresh(mCurrentMobileWorkItem);
			doAction(MobileWorkItem.STATUS_RESOURCE_STARTED, MobileWorkItem.STATUS_EXECUTING,
					"Saved");

			finishAndReturnResult(WorkItemActivity.UPDATE);

		} catch (SQLException e) {
			Log.e(TAG, e.getMessage(), e.fillInStackTrace());
		}
	}

	/**
	 * Completes a work item and finishes the activity.
	 */
	private void complete() {

		try {
			Log.d(TAG, "Refreshing: " + mCurrentMobileWorkItem.getID());

			this.mMobileWorkItemDao.refresh(mCurrentMobileWorkItem);

			// if item is suspended -> unsuspend and complete
			if (mCurrentMobileWorkItem.getResourceStatus().equals("Suspended")) {
				mCurrentMobileWorkItem.appendSyncAction(YConnectionService.SYNC_ACTION_UNSUSPEND);
			}

			mCurrentMobileWorkItem.setResourceStatus(MobileWorkItem.STATUS_COMPLETE);
			mCurrentMobileWorkItem.setState(State.MODIFIED);

			Log.d(TAG, "WorkItem-Offer completed: " + mCurrentMobileWorkItem.getID());
			Log.d(TAG, "Completed Wi data: " + mCurrentMobileWorkItem.getUpdatedDataString());

			if (!mCurrentMobileWorkItem.getSyncQueue()
					.contains(YConnectionService.SYNC_ACTION_SAVE)) {
				mCurrentMobileWorkItem.appendSyncAction(YConnectionService.SYNC_ACTION_SAVE);
			}
			mCurrentMobileWorkItem.appendSyncAction("Completed");
			Log.d(TAG, "Completed WI syncQueue: " + mCurrentMobileWorkItem.getSyncQueue());

			mMobileWorkItemDao.update(mCurrentMobileWorkItem);
		} catch (SQLException e) {
			Log.e(TAG, e.getMessage(), e.fillInStackTrace());
		}

		finishAndReturnResult(WorkItemActivity.COMPLETE);
	}

	/**
	 * Unsuspends a work item and finishes the activity.
	 */
	private void unsuspend() {

		doAction(MobileWorkItem.STATUS_RESOURCE_STARTED, null, "Unsuspended");

		finishAndReturnResult(WorkItemActivity.UNSUSPEND);
	}

	/**
	 * Process work item actions.
	 * 
	 * @param resourceStatus
	 *            The work item resource status.
	 * @param status
	 *            The work item status.
	 * @param action
	 *            The action to process.
	 */
	private void doAction(String resourceStatus, String status, String action) {
		try {
			mCurrentMobileWorkItem.setResourceStatus(resourceStatus);
			if (status != null && status.length() > 0) {
				mCurrentMobileWorkItem.setStatus(status);
			}
			Log.d(TAG, "WorkItem " + action + " " + mCurrentMobileWorkItem.getID());
			mCurrentMobileWorkItem.setState(State.MODIFIED);
			mCurrentMobileWorkItem.appendSyncAction(action);

			mMobileWorkItemDao.update(mCurrentMobileWorkItem);

		} catch (SQLException e) {
			Log.e(TAG, e.getMessage(), e.fillInStackTrace());
		}
	}

	/**
	 * Finishes the activity and returns activity results.
	 * 
	 * @param resultId
	 *            The result code.
	 */
	private void finishAndReturnResult(int resultId) {

		if (resultId != Activity.RESULT_CANCELED) {
			if (mLiveSyncEnabled) {
				liveSyncWorkitem();
			}
		}

		Intent in = new Intent();
		setResult(resultId, in);
		finish();
	}

	/**
	 * Invokes a live sync of the current work item.
	 */
	private void liveSyncWorkitem() {

		mDialog = ProgressDialog.show(this, "Please wait..", "Live-Sync in action", true, false);

		Handler handler = new Handler();
		handler.post(new Runnable() {
			@Override
			public void run() {
				if (mYConnectionService != null) {
					mYConnectionService.liveSyncWorkitem(mCurrentMobileWorkItem);
				}
				mDialog.dismiss();
				setProgressBarIndeterminateVisibility(false);
			}
		});
	}
}
