/**
 * 
 */
package de.msponer.android.yac.activities.workqueues;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import android.app.ListActivity;
import android.content.Intent;
import android.os.Bundle;
import android.text.format.DateFormat;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.ListView;
import android.widget.TextView;

import com.j256.ormlite.android.apptools.OpenHelperManager;
import com.j256.ormlite.android.apptools.OrmLiteSqliteOpenHelper;
import com.j256.ormlite.dao.Dao;
import com.j256.ormlite.stmt.PreparedQuery;
import com.j256.ormlite.stmt.QueryBuilder;

import de.msponer.android.yac.R;
import de.msponer.android.yac.adapter.SectionedWorkListAdapter;
import de.msponer.android.yac.adapter.WorkItemAdapter;
import de.msponer.android.yac.model.MobileWorkItem;

/**
 * (Super-)Activity for YAC Work-Queues. Used by all work queue tabs and by the
 * SearchResultActivity.
 * 
 * @author msponer
 * 
 */
public class WorkListActivity extends ListActivity {

	/**
	 * The application tag.
	 */
	private static final String TAG = "WorkListActivity";

	/**
	 * The list of work items.
	 */
	private List<MobileWorkItem> mWorkitemList;

	/**
	 * The list view that is used to display the work items.
	 */
	private ListView mWorkqueueListView;

	/**
	 * The request code.
	 */
	private int mRequestCode;

	/**
	 * The selected work item.
	 */
	private MobileWorkItem mSelectedItem;

	/**
	 * The mobile work item dao.
	 */
	private Dao<MobileWorkItem, Integer> mMobileWorkItemDao;

	/**
	 * A search pattern.
	 */
	private String mSearchPattern;

	/**
	 * The sectioned work list adapter.
	 */
	private SectionedWorkListAdapter mWorkItemAdapter;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);

		setContentView(R.layout.list_view_workqueue);

		try {
			OrmLiteSqliteOpenHelper dbHelper = OpenHelperManager.getHelper(this);
			this.mMobileWorkItemDao = dbHelper.getDao(MobileWorkItem.class);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	protected void onResume() {
		try {
			setupList();
		} catch (Exception e) {
			e.printStackTrace();
		}
		super.onResume();
	}

	@Override
	protected void onDestroy() {
		OpenHelperManager.release();
		super.onDestroy();
	}

	/**
	 * Sets up the work queue (list).
	 * 
	 * @throws Exception
	 *             Database or SQL exception.
	 */
	private void setupList() throws Exception {
		// Query DB for WorkItems
		if (mSearchPattern.equals("Offered") || mSearchPattern.equals("Allocated")
				|| mSearchPattern.equals("Started") || mSearchPattern.equals("Suspended")) {

			// Search for Tab-Content
			this.mWorkitemList = mMobileWorkItemDao.queryForEq("mResourceStatus", mSearchPattern);
		} else {

			// Search for Pattern
			QueryBuilder<MobileWorkItem, Integer> queryBuilder = mMobileWorkItemDao.queryBuilder();

			queryBuilder.where().like("mTaskName", mSearchPattern).or()
					.like("mDataListString", mSearchPattern).or()
					.like("mDataListUpdatedString", mSearchPattern);

			PreparedQuery<MobileWorkItem> query = queryBuilder.prepare();

			this.mWorkitemList = mMobileWorkItemDao.query(query);
		}

		if (mWorkitemList != null && mWorkitemList.size() > 0) {
			Log.d(TAG, mSearchPattern + " size: " + String.valueOf(mWorkitemList.size()));

			Log.d(TAG, "Sorting WorkitemList ...");
			Collections.sort(mWorkitemList);

			this.mWorkItemAdapter = createWorkItemAdapter();

			setListAdapter(this.mWorkItemAdapter);

			mWorkItemAdapter.notifyDataSetChanged();

			this.mWorkqueueListView = getListView();

			this.mWorkqueueListView.setTextFilterEnabled(true);
		}
	}

	/**
	 * Removes the selected work item from the queue.
	 */
	public void removeSelectedWorkItem() {

		this.mWorkitemList.remove(mSelectedItem);

		this.mWorkItemAdapter = createWorkItemAdapter();
		this.setListAdapter(this.mWorkItemAdapter);

		if (mWorkItemAdapter != null) {
			this.mWorkItemAdapter.notifyDataSetChanged();
		}

		this.mSelectedItem = null;
	}

	/**
	 * Initializes the sectioned work list adapter. Work item are sorted by date
	 * and grouped by start day.
	 * 
	 * @return The sectioned work list apdapter.
	 */
	public SectionedWorkListAdapter createWorkItemAdapter() {

		if (this.mWorkitemList == null || this.mWorkitemList.size() <= 0) {
			return null;
		}

		this.mWorkItemAdapter = new SectionedWorkListAdapter() {
			@Override
			protected View getHeaderView(String caption, int index, View convertView,
					ViewGroup parent) {
				TextView result = (TextView) convertView;

				if (convertView == null) {
					result = (TextView) getLayoutInflater()
							.inflate(R.layout.list_view_header, null);
				}

				result.setText(caption);

				return (result);
			}
		};

		List<List<MobileWorkItem>> sectionLists = new ArrayList<List<MobileWorkItem>>();

		List<MobileWorkItem> tempList = new ArrayList<MobileWorkItem>();
		tempList.add(mWorkitemList.get(0));
		int prevIndex = 0;

		for (int i = 1; i < mWorkitemList.size(); i++) {
			MobileWorkItem item = mWorkitemList.get(i);

			Date itemDate = item.getEnablementDate();
			Date prevDate = tempList.get(prevIndex).getEnablementDate();

			if (itemDate.getDate() == prevDate.getDate()
					&& itemDate.getMonth() == prevDate.getMonth()
					&& itemDate.getYear() == prevDate.getYear()) {
				// Add item to existing list
				tempList.add(item);
				prevIndex++;
			} else {
				// Create a new list to start a new section
				sectionLists.add(tempList);
				tempList = new ArrayList<MobileWorkItem>();
				tempList.add(item);
				prevIndex = 0;
			}
		}

		if (!sectionLists.contains(tempList)) {
			sectionLists.add(tempList);
		}
		for (List<MobileWorkItem> list : sectionLists) {
			mWorkItemAdapter.addSection(
					DateFormat.getLongDateFormat(this).format(list.get(0).getEnablementDate()),
					new WorkItemAdapter(this, R.layout.list_item_queues, list));
		}

		return mWorkItemAdapter;
	}

	/**
	 * Starts the WorkItemActivity and passes the specified intent actions.
	 * 
	 * @param intentAction
	 *            The intent action.
	 */
	public void startWorkItemActivity(String intentAction) {

		Intent workItemIntent = new Intent(this, WorkItemActivity.class);

		Log.d(TAG, "Selected WorkItem-ID: " + this.mSelectedItem.getID());
		Log.d(TAG, "Workitem Intent-Action: " + intentAction);
		workItemIntent.putExtra("PRIMARY_KEY", this.mSelectedItem.getPrimaryKey());

		if (intentAction != null && intentAction.length() > 0) {
			workItemIntent.setAction(intentAction);
		}

		if (getParent() != null) {
			getParent().setProgressBarIndeterminateVisibility(true);
		} else {
			setProgressBarIndeterminateVisibility(true);
		}
		
		startActivityForResult(workItemIntent, getRequestCode());
	}

	/**
	 * Return the current request code.
	 * 
	 * @return The request code.
	 */
	public int getRequestCode() {
		return mRequestCode;
	}

	/**
	 * Sets the current request code.
	 * 
	 * @param requestCode
	 *            The request code to set.
	 */
	public void setRequestCode(int requestCode) {
		this.mRequestCode = requestCode;
	}

	/**
	 * Returns the selected work item.
	 * 
	 * @return The selected work item.
	 */
	public MobileWorkItem getSelectedItem() {
		return mSelectedItem;
	}

	/**
	 * Sets the selected work item.
	 * 
	 * @param selectedItem
	 *            The work item to select.
	 */
	public void setSelectedItem(MobileWorkItem selectedItem) {
		this.mSelectedItem = selectedItem;
	}

	/**
	 * Returns the mobile work item dao.
	 * 
	 * @return The current mobile work item.
	 */
	public Dao<MobileWorkItem, Integer> getMobileWorkItemDao() {
		return mMobileWorkItemDao;
	}

	/**
	 * Sets the current mobile work item dao.
	 * 
	 * @param mobileWorkItemDao
	 *            The mobile work item dao to set.
	 */
	public void setMobileWorkItemDao(Dao<MobileWorkItem, Integer> mobileWorkItemDao) {
		this.mMobileWorkItemDao = mobileWorkItemDao;
	}

	/**
	 * Sets the current search pattern.
	 * 
	 * @param searchPattern
	 *            The search pattern to set.
	 */
	public void setSearchPattern(String searchPattern) {
		this.mSearchPattern = searchPattern;
	}

	/**
	 * Returns the current search pattern.
	 * 
	 * @return The current search pattern.
	 */
	public String getSearchPattern() {
		return mSearchPattern;
	}

	/**
	 * Sets the sectioned work item adapter.
	 * 
	 * @param workItemAdapter
	 *            The sectioned work item adpater.
	 */
	public void setWorkItemAdapter(SectionedWorkListAdapter workItemAdapter) {
		this.mWorkItemAdapter = workItemAdapter;
	}

	/**
	 * Return the sectioned work item adapter.
	 * 
	 * @return The sectioned work item adapter.
	 */
	public SectionedWorkListAdapter getWorkItemAdapter() {
		return mWorkItemAdapter;
	}

	/**
	 * Sets the current work item list.
	 * 
	 * @param workitemList
	 *            The current work item list.
	 */
	public void setWorkitemList(List<MobileWorkItem> workitemList) {
		this.mWorkitemList = workitemList;
	}

	/**
	 * Returns the current work item list.
	 * 
	 * @return The current work item list.
	 */
	public List<MobileWorkItem> getWorkitemList() {
		return mWorkitemList;
	}

	/**
	 * Sets the current work queue lis view.
	 * 
	 * @param workqueueListView
	 *            The work queue list view to set.
	 */
	public void setWorkqueueListView(ListView workqueueListView) {
		this.mWorkqueueListView = workqueueListView;
	}

	/**
	 * Returns the current work queue list view.
	 * 
	 * @return The current work queue list view.
	 */
	public ListView getWorkqueueListView() {
		return mWorkqueueListView;
	}
}
