/**
 * 
 */
package de.msponer.android.yac.adapter;

import java.util.ArrayList;
import java.util.List;

import android.view.View;
import android.view.ViewGroup;
import android.widget.Adapter;
import android.widget.BaseAdapter;

/**
 * A sectioned work item adapter. Based on ideas presented in
 * "The Busy Coder's Guide to Advanced Android Development"
 * (see http://commonsware.com/AdvAndroid)
 * 
 * @author msponer
 * 
 */
public abstract class SectionedWorkListAdapter extends BaseAdapter {

	/**
	 * The header view.
	 * 
	 * @param caption The caption.
	 * @param index The index.
	 * @param convertView The view to convert.
	 * @param parent The parent view.
	 * @return The resulting header view.
	 */
	protected abstract View getHeaderView(String caption, int index, View convertView,
			ViewGroup parent);

	/**
	 * The list of work item sections.
	 */
	private List<Section> mSections = new ArrayList<Section>();
	
	/**
	 * The section type header.
	 */
	private static int sTypeSectionHeader = 0;

	/**
	 * Constructor.
	 */
	public SectionedWorkListAdapter() {
		super();
	}

	/**
	 * Adds a new section.
	 * @param caption The section caption.
	 * @param adapter The adapter to add.
	 */
	public void addSection(String caption, Adapter adapter) {
		mSections.add(new Section(caption, adapter));
	}

	/**
	 * Returns item object located at the given position.
	 * @param position The position.
	 * @return The item object.
	 */
	public Object getItem(int position) {

		for (Section section : this.mSections) {
			if (position == 0) {
				return (section);
			}
			int size = section.getAdapter().getCount() + 1;

			if (position < size) {
				return (section.getAdapter().getItem(position - 1));
			}

			position -= size;
		}

		return (null);
	}

	/**
	 * The section count.
	 * @return The section count.
	 */
	public int getCount() {
		int total = 0;

		for (Section section : this.mSections) {
			total += section.getAdapter().getCount() + 1;
		}

		return (total);
	}

	/**
	 * The view type count.
	 * @return The view type count.
	 */
	public int getViewTypeCount() {
		int total = 1;

		for (Section section : this.mSections) {
			total += section.getAdapter().getViewTypeCount();
		}

		return (total);
	}

	/**
	 * The view type count located at the given position.
	 * @param position The position.
	 * @return The view type count.
	 */
	public int getItemViewType(int position) {
		int typeOffset = sTypeSectionHeader + 1;

		for (Section section : this.mSections) {
			if (position == 0) {
				return (sTypeSectionHeader);
			}

			int size = section.getAdapter().getCount() + 1;

			if (position < size) {
				return (typeOffset + section.getAdapter().getItemViewType(position - 1));
			}

			position -= size;
			typeOffset += section.getAdapter().getViewTypeCount();
		}

		return (-1);
	}

	/**
	 * Returns true, if all items are selectable.
	 * @return true, if all items are selectable.
	 */
	public boolean areAllItemsSelectable() {
		return (false);
	}
	
	/**
	 * Returns true if item at the given position is enabled.
	 * @param position The position to check.
	 * @return true if item at the given position is enabled.
	 */
	public boolean isEnabled(int position) {
		return (getItemViewType(position) != sTypeSectionHeader);
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		int sectionIndex = 0;

		for (Section section : this.mSections) {
			if (position == 0) {
				return (getHeaderView(section.getCaption(), sectionIndex, convertView, parent));
			}

			int size = section.getAdapter().getCount() + 1;

			if (position < size) {
				return (section.getAdapter().getView(position - 1, convertView, parent));
			}

			position -= size;
			sectionIndex++;
		}

		return (null);
	}

	@Override
	public long getItemId(int position) {
		return (position);
	}

	/**
	 * Returns the section adapter located at the given position. 
	 * @param section The section number to return.
	 * @return The section adapter.
	 */
	public Adapter getSectionAdapter(int section) {
		return this.mSections.get(section).getAdapter();
	}

	/**
	 * A section class.
	 * 
	 * @author msponer
	 *
	 */
	class Section {
		/**
		 * The section caption.
		 */
		private String mCaption;
		
		/**
		 * The section adapter.
		 */
		private Adapter mAdapter;

		/**
		 * Constructor setting the required fields.
		 * @param caption The caption.
		 * @param adapter The adapter.
		 */
		Section(String caption, Adapter adapter) {
			this.setCaption(caption);
			this.setAdapter(adapter);
		}

		/**
		 * Sets the caption.
		 * @param caption The caption to set.
		 */
		public void setCaption(String caption) {
			this.mCaption = caption;
		}

		/**
		 * Return the caption.
		 * @return The section caption.
		 */
		public String getCaption() {
			return mCaption;
		}

		/**
		 * Sets the adapter.
		 * @param adapter The adapter to set.
		 */
		public void setAdapter(Adapter adapter) {
			this.mAdapter = adapter;
		}

		/**
		 * Return the adapter.
		 * @return The adapter.
		 */
		public Adapter getAdapter() {
			return mAdapter;
		}
	}

}
