/**
 * 
 */
package de.msponer.android.yac.db;

import java.util.List;

import org.jdom.Element;
import org.yawlfoundation.yawl.util.JDOMUtil;

import android.content.Context;
import android.util.Log;

import com.j256.ormlite.android.apptools.OpenHelperManager;
import com.j256.ormlite.dao.Dao;
import com.j256.ormlite.stmt.PreparedQuery;
import com.j256.ormlite.stmt.QueryBuilder;
import com.j256.ormlite.stmt.Where;

import de.msponer.android.yac.model.MobileWorkItem;

/**
 * Helper class to generate ORMLite queries.
 * 
 * @author msponer
 * 
 */
public class QueryHelper {

	/**
	 * The class tag.
	 */
	private static final String TAG = "QueryHelper";

	/**
	 * The application context.
	 */
	private Context mContext;

	/**
	 * The mobile work item dao.
	 */
	private Dao<MobileWorkItem, Integer> mMobileWorkItemDao;

	/**
	 * Constructor setting the application context.
	 * 
	 * @param context
	 *            The app context.
	 */
	public QueryHelper(Context context) {
		this.mContext = context;
	}

	/**
	 * Queries the database for all localized work items and returns the
	 * resulting list.
	 * 
	 * @param lonKey
	 *            The key for longitude
	 * @param latKey
	 *            The key for latitude
	 * @return A list of all localized work items.
	 * @throws Exception
	 *             Errors during query.
	 */
	public List<MobileWorkItem> queryForAllLocalizedWorkitems(String lonKey, String latKey)
			throws Exception {

		Log.d(TAG, "Looking up keys: " + lonKey + ", " + latKey);

		List<MobileWorkItem> resultList;

		this.mMobileWorkItemDao = OpenHelperManager.getHelper(this.mContext).getDao(
				MobileWorkItem.class);

		// Search for Pattern
		QueryBuilder<MobileWorkItem, Integer> queryBuilder = mMobileWorkItemDao.queryBuilder();

		Where<MobileWorkItem, Integer> where = queryBuilder.where();

		where.and(where.like("mDataSchemaString", "%" + lonKey + "%"),
				where.like("mDataSchemaString", "%" + latKey + "%"));

		PreparedQuery<MobileWorkItem> query = queryBuilder.prepare();

		resultList = mMobileWorkItemDao.query(query);

		if (resultList != null && resultList.size() > 0) {
			Log.d(TAG, "Size: " + String.valueOf(resultList.size()));

			Log.d(TAG, "Scanning for coordinates ...");

			for (MobileWorkItem workitem : resultList) {

				Log.d(TAG, "Workitem: " + workitem.getID());
				Element data = null;

				if (workitem.getUpdatedDataString() != null
						&& workitem.getUpdatedDataString().length() > 0) {
					Log.d(TAG, "Using updated-datalist to determine coordinates");
					data = JDOMUtil.stringToElement(workitem.getUpdatedDataString());
				} else if (workitem.getDataListString() != null
						&& workitem.getDataListString().length() > 0) {
					Log.d(TAG, "Using datalist to determine coordinates");
					data = JDOMUtil.stringToElement(workitem.getDataListString());
				} else {
					Log.d(TAG, "Using case data to determine coordinates");
					data = JDOMUtil.stringToElement(workitem.getCaseDataString());
				}

				if (data != null) {
					determineCoordinates(workitem, data, lonKey, latKey);
				} else {
					workitem.setLongitude("");
					workitem.setLatitude("");
				}
			}
		}

		return resultList;

	}

	/**
	 * Determines coordinates (latitute, longitude) for a localized work item.
	 * 
	 * @param workitem
	 *            The localized work item.
	 * @param data
	 *            The XML data elements to scan through.
	 * @param lonKey
	 *            The longitude key
	 * @param latKey
	 *            The latitude key
	 */
	private void determineCoordinates(MobileWorkItem workitem, Element data, String lonKey,
			String latKey) {

		Element longitude = data.getChild(lonKey);
		Element latitude = data.getChild(latKey);

		if (longitude != null && latitude != null) {

			Log.d(TAG, "Found Longitude" + longitude.getValue());
			Log.d(TAG, "Found Latitude" + latitude.getValue());

			if (isDouble(longitude.getValue()) && isDouble(latitude.getValue())) {
				workitem.setLongitude(longitude.getValue());
				workitem.setLatitude(latitude.getValue());
			}
		} else {

			Log.d(TAG, "No coordinates found, scanning children ...");

			if (data.getChildren() != null) {
				for (Object childObject : data.getChildren()) {
					Element child = (Element) childObject;
					determineCoordinates(workitem, child, lonKey, latKey);
				}
			}
		}
	}

	/**
	 * Is the given string a valid double value?
	 * 
	 * @param value
	 *            The String to check.
	 * @return true, if the given string is a valid double value.
	 */
	private boolean isDouble(String value) {

		try {
			Double.parseDouble(value);
			return true;
		} catch (Exception e) {
			return false;
		}
	}
}