/**
 * 
 */
package de.msponer.android.yac.model;

import java.io.Serializable;
import java.util.Date;

import org.jdom.Element;
import org.yawlfoundation.yawl.engine.interfce.WorkItemRecord;
import org.yawlfoundation.yawl.util.JDOMUtil;
import org.yawlfoundation.yawl.util.StringUtil;

import com.j256.ormlite.field.DatabaseField;
import com.j256.ormlite.table.DatabaseTable;

/**
 * Model-Class for mobile work items. Corresponding YAWl class is
 * org.yawlfoundation.yawl.engine.interfce.WorkItemRecord. Extension added to
 * support special mobile functionalities (current position, distances, etc.)
 * 
 * @author msponer
 * 
 */
@DatabaseTable
public class MobileWorkItem implements Comparable<MobileWorkItem>, Serializable {

	/**
	 * A unique serial version id.
	 */
	private static final long serialVersionUID = 5540723928109628846L;

	/**
	 * Constant for status enabled.
	 */
	public static final String STATUS_ENABLED = "Enabled";

	/**
	 * Constant for status executing.
	 */
	public static final String STATUS_EXECUTING = "Executing";

	/**
	 * Constant for status complete.
	 */
	public static final String STATUS_COMPLETE = "Complete";

	/**
	 * Constant for status suspended.
	 */
	public static final String STATUS_SUSPENDED = "Suspended";

	/**
	 * Constant for resource status offered.
	 */
	public static final String STATUS_RESOURCE_OFFERED = "Offered";

	/**
	 * Constant for resource status allocated.
	 */
	public static final String STATUS_RESOURCE_ALLOCATED = "Allocated";

	/**
	 * Constant for resource status started.
	 */
	public static final String STATUS_RESOURCE_STARTED = "Started";

	/**
	 * Constant for resource status suspended.
	 */
	public static final String STATUS_RESOURCE_SUSPENDED = "Suspended";

	/**
	 * Constant for resource status unoffered.
	 */
	public static final String STATUS_RESOURCE_UNOFFERED = "Unoffered";

	/**
	 * Constant for resource status unresourced.
	 */
	public static final String STATUS_RESOURCE_UNRESOURCED = "Unresourced";

	/**
	 * The primary key of the mobile work item.
	 */
	@DatabaseField(generatedId = true)
	private int mPrimaryKey;

	/**
	 * The specification identifier.
	 */
	@DatabaseField
	private String mSpecIdentifier;

	/**
	 * The specification version.
	 */
	@DatabaseField
	private String mSpecVersion = "0.1";

	/**
	 * The specification URI.
	 */
	@DatabaseField
	private String mSpecURI;

	/**
	 * The case id.
	 */
	@DatabaseField
	private String mCaseID;

	/**
	 * The task id.
	 */
	@DatabaseField
	private String mTaskID;

	/**
	 * The task name.
	 */
	@DatabaseField
	private String mTaskName;

	/**
	 * The enablement time string.
	 */
	@DatabaseField
	private String mEnablementTimeString;

	/**
	 * The enablement time string in ms.
	 */
	@DatabaseField
	private String mEnablementTimeMsString;

	/**
	 * The converted enablement date.
	 */
	@DatabaseField
	private Date mEnablementDate;

	/**
	 * The current status.
	 */
	@DatabaseField
	private String mStatus;

	/**
	 * The current resource status.
	 */
	@DatabaseField
	private String mResourceStatus = STATUS_RESOURCE_UNRESOURCED;

	// initial data params and values

	/**
	 * The initial data list.
	 */
	private Element mDataList;

	/**
	 * The initial data list string.
	 */
	@DatabaseField
	private String mDataListString;

	/**
	 * The updated data list.
	 */
	private Element mDataListUpdated;

	/**
	 * The updated data list string.
	 */
	@DatabaseField
	private String mDataListUpdatedString;

	/**
	 * The data schema (xsd).
	 */
	private Element mDataSchema;

	/**
	 * The data schema string.
	 */
	@DatabaseField
	private String mDataSchemaString;

	/**
	 * The case data.
	 */
	private Element mCaseData;

	/**
	 * The case data string.
	 */
	@DatabaseField
	private String mCaseDataString;

	/**
	 * The local state.
	 */
	@DatabaseField
	private State mState = State.UNMODIFIED;

	/**
	 * The sync queue.
	 */
	@DatabaseField
	private String mSyncQueue = "";

	/**
	 * The work item's longitude.
	 */
	@DatabaseField
	private String mLongitude = "";

	/**
	 * The work item's latitude.
	 */
	@DatabaseField
	private String mLatitude = "";

	/**
	 * The distance to the current position.
	 */
	@DatabaseField
	private float mDistanceToPosition = -1;

	/**
	 * Instantiates a new mobile work item.
	 */
	public MobileWorkItem() {
	}

	/**
	 * Instantiates a new mobile work item.
	 * 
	 * @param caseID
	 *            the case id
	 * @param taskID
	 *            the task id
	 * @param specURI
	 *            the spec uri
	 * @param enablementTime
	 *            the enablement time
	 * @param status
	 *            the status
	 */
	public MobileWorkItem(String caseID, String taskID, String specURI, String enablementTime,
			String status) {
		mTaskID = taskID;
		mCaseID = caseID;
		mSpecURI = specURI;
		mEnablementTimeString = enablementTime;
		mStatus = status;
	}

	/**
	 * Instantiates a new mobile work item.
	 * 
	 * @param wir
	 *            the work item record
	 */
	public MobileWorkItem(WorkItemRecord wir) {
		mCaseID = wir.getCaseID();
		mEnablementTimeString = wir.getEnablementTime();
		mEnablementTimeMsString = wir.getEnablementTimeMs();
		mResourceStatus = wir.getResourceStatus();
		mSpecIdentifier = wir.getSpecIdentifier();
		mSpecURI = wir.getSpecURI();
		mSpecVersion = wir.getSpecVersion();
		mStatus = wir.getStatus();
		mTaskID = wir.getTaskID();
		mTaskName = wir.getTaskName();

		setDataList(wir.getDataList());
		setUpdatedData(wir.getUpdatedData());

		mEnablementDate = new Date(Long.parseLong(this.mEnablementTimeMsString));
	}

	/**
	 * Restores the data list.
	 */
	public void restoreDataList() {
		if (mDataListString != null) {
			mDataList = JDOMUtil.stringToElement(mDataListString);
		}
		if (mDataListUpdatedString != null) {
			mDataListUpdated = JDOMUtil.stringToElement(mDataListUpdatedString);
		}
	}

	/**
	 * Sets the spec identifier.
	 * 
	 * @param id
	 *            the new spec identifier
	 */
	public void setSpecIdentifier(String id) {
		mSpecIdentifier = id;
	}

	/**
	 * Sets the spec version.
	 * 
	 * @param version
	 *            the new spec version
	 */
	public void setSpecVersion(String version) {
		mSpecVersion = version;
	}

	/**
	 * Sets the spec uri.
	 * 
	 * @param uri
	 *            the new spec uri
	 */
	public void setSpecURI(String uri) {
		mSpecURI = uri;
	}

	/**
	 * Sets the case id.
	 * 
	 * @param caseID
	 *            the new case id
	 */
	public void setCaseID(String caseID) {
		this.mCaseID = caseID;
	}

	/**
	 * Sets the task id.
	 * 
	 * @param taskID
	 *            the new task id
	 */
	public void setTaskID(String taskID) {
		this.mTaskID = taskID;
	}

	/**
	 * Sets the task name.
	 * 
	 * @param name
	 *            the new task name
	 */
	public void setTaskName(String name) {
		mTaskName = name;
	}

	/**
	 * Sets the enablement time string.
	 * 
	 * @param time
	 *            the new enablement time string
	 */
	public void setEnablementTimeString(String time) {
		mEnablementTimeString = time;
	}

	/**
	 * Sets the enablement time ms.
	 * 
	 * @param time
	 *            the new enablement time ms
	 */
	public void setEnablementTimeMs(String time) {
		mEnablementTimeMsString = time;
	}

	/**
	 * Sets the status.
	 * 
	 * @param status
	 *            the new status
	 */
	public void setStatus(String status) {
		this.mStatus = status;
	}

	/**
	 * Sets the resource status.
	 * 
	 * @param status
	 *            the new resource status
	 */
	public void setResourceStatus(String status) {
		mResourceStatus = status;
	}

	/**
	 * Sets the data list.
	 * 
	 * @param dataList
	 *            the new data list
	 */
	public void setDataList(Element dataList) {
		this.mDataList = dataList;
		this.mDataListString = JDOMUtil.elementToStringDump(dataList);
	}

	/**
	 * Sets the data list string.
	 * 
	 * @param dataStr
	 *            the new data list string
	 */
	public void setDataListString(String dataStr) {
		this.mDataListString = dataStr;
		this.mDataList = JDOMUtil.stringToElement(dataStr);
	}

	/**
	 * Sets the updated data.
	 * 
	 * @param dataListUpdated
	 *            the new updated data
	 */
	public void setUpdatedData(Element dataListUpdated) {
		this.mDataListUpdated = dataListUpdated;
		this.mDataListUpdatedString = JDOMUtil.elementToString(dataListUpdated);
	}

	/**
	 * Sets the updated data list string.
	 * 
	 * @param updatedDataStr
	 *            the new updated data list string
	 */
	public void setUpdatedDataListString(String updatedDataStr) {
		this.mDataListUpdatedString = updatedDataStr;
		this.mDataListUpdated = JDOMUtil.stringToElement(updatedDataStr);
	}

	/**
	 * Sets the data schema.
	 * 
	 * @param dataSchema
	 *            the new data schema
	 */
	public void setDataSchema(Element dataSchema) {
		this.mDataSchema = dataSchema;
		this.mDataSchemaString = JDOMUtil.elementToString(dataSchema);
	}

	/**
	 * Sets the data schema string.
	 * 
	 * @param dataSchemaString
	 *            the new data schema string
	 */
	public void setDataSchemaString(String dataSchemaString) {
		this.mDataSchemaString = dataSchemaString;
		this.mDataSchema = JDOMUtil.stringToElement(dataSchemaString);
	}

	/**
	 * Returns the spec identifier.
	 * 
	 * @return the spec identifier
	 */
	public String getSpecIdentifier() {
		return mSpecIdentifier;
	}

	/**
	 * Returns the spec version.
	 * 
	 * @return the spec version
	 */
	public String getSpecVersion() {
		return mSpecVersion;
	}

	/**
	 * Returns the spec uri.
	 * 
	 * @return the spec uri
	 */
	public String getSpecURI() {
		return mSpecURI;
	}

	/**
	 * Returns the case id.
	 * 
	 * @return the case id
	 */
	public String getCaseID() {
		return mCaseID;
	}

	/**
	 * Returns the task id.
	 * 
	 * @return the task id
	 */
	public String getTaskID() {
		return mTaskID;
	}

	/**
	 * Returns the iD.
	 * 
	 * @return the iD
	 */
	public String getID() {
		return mCaseID + ":" + mTaskID;
	}

	/**
	 * Returns the enablement time string.
	 * 
	 * @return the enablement time string
	 */
	public String getEnablementTimeString() {
		return mEnablementTimeString;
	}

	/**
	 * Returns the enablement time ms.
	 * 
	 * @return the enablement time ms
	 */
	public String getEnablementTimeMs() {
		return mEnablementTimeMsString;
	}

	/**
	 * Returns the status.
	 * 
	 * @return the status
	 */
	public String getStatus() {
		return mStatus;
	}

	/**
	 * Returns the resource status.
	 * 
	 * @return the resource status
	 */
	public String getResourceStatus() {
		return mResourceStatus;
	}

	/**
	 * Returns the data list.
	 * 
	 * @return the data list
	 */
	public Element getDataList() {
		return mDataList;
	}

	/**
	 * Returns the data list string.
	 * 
	 * @return the data list string
	 */
	public String getDataListString() {
		return mDataListString;
	}

	/**
	 * Returns the updated data.
	 * 
	 * @return the updated data
	 */
	public Element getUpdatedData() {
		return mDataListUpdated;
	}

	/**
	 * Returns the updated data string.
	 * 
	 * @return the updated data string
	 */
	public String getUpdatedDataString() {
		return mDataListUpdatedString;
	}

	/**
	 * Returns the iD for display.
	 * 
	 * @return the iD for display
	 */
	public String getIDForDisplay() {
		return mCaseID + ":" + mTaskName;
	}

	/**
	 * Returns the task name.
	 * 
	 * @return the task name
	 */
	public String getTaskName() {
		return mTaskName;
	}

	/**
	 * Returns the root case id.
	 * 
	 * @return the root case id
	 */
	public String getRootCaseID() {
		String result = mCaseID;
		if ((mCaseID != null) && (mCaseID.indexOf(".") > 0)) {
			result = mCaseID.split("\\.")[0];
		}
		return result;
	}

	/**
	 * Checks for live status.
	 * 
	 * @return true, if successful
	 */
	public boolean hasLiveStatus() {
		return mStatus.equals(STATUS_ENABLED) || mStatus.equals(STATUS_EXECUTING);
	}

	/**
	 * Checks for status.
	 * 
	 * @param status
	 *            the status
	 * @return true, if successful
	 */
	public boolean hasStatus(String status) {
		return (status != null) && status.equals(status);
	}

	/**
	 * Checks for resource status.
	 * 
	 * @param status
	 *            the status
	 * @return true, if successful
	 */
	public boolean hasResourceStatus(String status) {
		return (mResourceStatus != null) && mResourceStatus.equals(status);
	}

	/**
	 * Returns the data schema.
	 * 
	 * @return the data schema
	 */
	public Element getDataSchema() {
		return mDataSchema;
	}

	/**
	 * Returns the data schema string.
	 * 
	 * @return the data schema string
	 */
	public String getDataSchemaString() {
		return mDataSchemaString;
	}

	/**
	 * To xml.
	 * 
	 * @return the string
	 */
	public String toXML() {
		StringBuilder xml = new StringBuilder("<workItemRecord>");
		xml.append(StringUtil.wrap(getID(), "id"))
				.append(StringUtil.wrap(mSpecVersion, "specversion"))
				.append(StringUtil.wrap(mSpecURI, "specuri"))
				.append(StringUtil.wrap(mCaseID, "caseid"))
				.append(StringUtil.wrap(mTaskID, "taskid"))
				.append(StringUtil.wrap(mTaskName, "taskname"))
				.append(StringUtil.wrap(mEnablementTimeString, "enablementTime"))
				.append(StringUtil.wrap(mEnablementTimeMsString, "enablementTimeMs"))
				.append(StringUtil.wrap(mStatus, "status"))
				.append(StringUtil.wrap(mResourceStatus, "resourceStatus"));

		if (mSpecIdentifier != null) {
			xml.append(StringUtil.wrap(mSpecIdentifier, "specidentifier"));
		}

		xml.append("<data>");
		if (mDataList != null) {
			xml.append(JDOMUtil.elementToStringDump(mDataList));
		}
		xml.append("</data>");
		xml.append("<updateddata>");

		if (mDataListUpdated != null) {
			JDOMUtil.elementToStringDump(mDataListUpdated);
		}
		xml.append("</updateddata>");
		xml.append("</workItemRecord>");

		return xml.toString();
	}

	/**
	 * Returns the primary key.
	 * 
	 * @return the primary key
	 */
	public int getPrimaryKey() {
		return mPrimaryKey;
	}

	/**
	 * Sets the primary key.
	 * 
	 * @param primaryKey
	 *            the new primary key
	 */
	public void setPrimaryKey(int primaryKey) {
		this.mPrimaryKey = primaryKey;
	}

	/**
	 * Sets the state.
	 * 
	 * @param state
	 *            the new state
	 */
	public void setState(State state) {
		this.mState = state;
	}

	/**
	 * Returns the state.
	 * 
	 * @return the state
	 */
	public State getState() {
		return mState;
	}

	/**
	 * Sets the sync queue.
	 * 
	 * @param syncQueue
	 *            the new sync queue
	 */
	public void setSyncQueue(String syncQueue) {
		this.mSyncQueue = syncQueue;
	}

	/**
	 * Returns the sync queue.
	 * 
	 * @return the sync queue
	 */
	public String getSyncQueue() {
		return mSyncQueue;
	}

	/**
	 * Append sync action.
	 * 
	 * @param actionString
	 *            the action string
	 */
	public void appendSyncAction(String actionString) {
		if (mSyncQueue == null) {
			mSyncQueue = "";
		}
		mSyncQueue += actionString + ",";
	}

	/**
	 * Removes the sync action.
	 * 
	 * @param actionString
	 *            the action string
	 */
	public void removeSyncAction(String actionString) {
		mSyncQueue.replace(actionString + ",", "");
	}

	/**
	 * Returns the wIR.
	 * 
	 * @return the wIR
	 */
	public WorkItemRecord getWIR() {

		WorkItemRecord wir = new WorkItemRecord();

		wir.setCaseID(mCaseID);
		wir.setEnablementTime(mEnablementTimeString);
		wir.setEnablementTimeMs(mEnablementTimeMsString);
		wir.setResourceStatus(mResourceStatus);
		wir.setSpecIdentifier(mSpecIdentifier);
		wir.setSpecURI(mSpecURI);
		wir.setSpecVersion(mSpecVersion);
		wir.setStatus(mStatus);
		wir.setTaskID(mTaskID);
		wir.setTaskName(mTaskName);
		wir.setDataList(mDataList);
		wir.setUpdatedData(mDataListUpdated);

		return wir;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(MobileWorkItem another) {

		if (this.mEnablementDate.getTime() > another.getEnablementDate().getTime()) {
			return -1;
		} else if (this.mEnablementDate.getTime() < another.getEnablementDate().getTime()) {
			return 1;
		}
		return 0;
	}

	/**
	 * Sets the enablement date.
	 * 
	 * @param enablementDate
	 *            the new enablement date
	 */
	public void setEnablementDate(Date enablementDate) {
		this.mEnablementDate = enablementDate;
	}

	/**
	 * Returns the enablement date.
	 * 
	 * @return the enablement date
	 */
	public Date getEnablementDate() {
		return mEnablementDate;
	}

	/**
	 * Sets the longitude.
	 * 
	 * @param longitude
	 *            the new longitude
	 */
	public void setLongitude(String longitude) {
		this.mLongitude = longitude;
	}

	/**
	 * Returns the longitude.
	 * 
	 * @return the longitude
	 */
	public String getLongitude() {
		return mLongitude;
	}

	/**
	 * Sets the latitude.
	 * 
	 * @param latitude
	 *            the new latitude
	 */
	public void setLatitude(String latitude) {
		this.mLatitude = latitude;
	}

	/**
	 * Returns the latitude.
	 * 
	 * @return the latitude
	 */
	public String getLatitude() {
		return mLatitude;
	}

	/**
	 * Sets the distance to position.
	 * 
	 * @param distanceToPosition
	 *            the new distance to position
	 */
	public void setDistanceToPosition(float distanceToPosition) {
		this.mDistanceToPosition = distanceToPosition;
	}

	/**
	 * Returns the distance to position.
	 * 
	 * @return the distance to position
	 */
	public float getDistanceToPosition() {
		return mDistanceToPosition;
	}

	/**
	 * Sets the case data.
	 * 
	 * @param caseData
	 *            the new case data
	 */
	public void setCaseData(Element caseData) {
		this.mCaseData = caseData;
		this.mCaseDataString = JDOMUtil.elementToString(caseData);
	}

	/**
	 * Returns the case data.
	 * 
	 * @return the case data
	 */
	public Element getCaseData() {
		return mCaseData;
	}

	/**
	 * Sets the case data string.
	 * 
	 * @param caseDataString
	 *            the new case data string
	 */
	public void setCaseDataString(String caseDataString) {
		this.mCaseDataString = caseDataString;
		this.mCaseData = JDOMUtil.stringToElement(caseDataString);
	}

	/**
	 * Returns the case data string.
	 * 
	 * @return the case data string
	 */
	public String getCaseDataString() {
		return mCaseDataString;
	}

}
