/**
 * 
 */
package de.msponer.android.yac.xml;

import java.util.ArrayList;
import java.util.List;

import android.util.Log;

/**
 * Class representing a work item XSD data element. A WorkItemElementEntry
 * contains several XSD attributes. Can have a father entry and list of
 * sub-entries. Corresponding YAWL class is:
 * org.yawlfoundation.yawl.resourcing.jsf.dynform.DynFormField
 * 
 * @author msponer
 */
public class WorkItemElementEntry implements Cloneable {
	/**
	 * The entry id.
	 */
	private int mId;

	/**
	 * The entry name.
	 */
	private String mName;

	/**
	 * The entry datatype.
	 */
	private String mDatatype;

	/**
	 * The entry value.
	 */
	private String mValue;

	/**
	 * The entry minoccur attribute.
	 */
	private long mMinoccurs = 1;

	/**
	 * The entry max occur attribute.
	 */
	private long mMaxoccurs = 1;

	/**
	 * The XSD level the entry is located at.
	 */
	private int mLevel;

	/**
	 * The order.
	 */
	private int mOrder;

	/**
	 * The occurrence count.
	 */
	private long mOccursCount;

	/**
	 * The group id.
	 */
	private String mGroupID;

	/**
	 * The parent entry.
	 */
	private WorkItemElementEntry mParent;

	/**
	 * The entry's child elements.
	 */
	private List<WorkItemElementEntry> mSubElementEntryList;

	/**
	 * The corresponding android view id.
	 */
	private int mViewId;

	/**
	 * The entry restrictions.
	 */
	private WorkItemElementRestriction mRestriction;

	/**
	 * Constructor setting name and child entries.
	 * 
	 * @param name
	 *            The entry's name.
	 * @param subElementEntryList
	 *            The entry's child elements.
	 */
	public WorkItemElementEntry(String name, List<WorkItemElementEntry> subElementEntryList) {
		this.mName = name;

		this.mSubElementEntryList = subElementEntryList;
		if (this.mSubElementEntryList != null) {
			for (WorkItemElementEntry entry : this.mSubElementEntryList) {
				entry.setParent(this);
			}
		}
	}

	/**
	 * Constructor setting name, datatype and value.
	 * 
	 * @param name
	 *            The entry's name.
	 * @param datatype
	 *            The entry's datatype.
	 * @param value
	 *            The entry's value.
	 */
	public WorkItemElementEntry(String name, String datatype, String value) {
		this.mName = name;
		this.mDatatype = datatype;
		this.mValue = value;
	}

	/**
	 * Sets the name.
	 * 
	 * @param name
	 *            the new name
	 */
	public void setName(String name) {
		this.mName = name;
	}

	/**
	 * Gets the name.
	 * 
	 * @return the name
	 */
	public String getName() {
		return mName;
	}

	/**
	 * Gets the datatype.
	 * 
	 * @return the datatype
	 */
	public String getDatatype() {
		return mDatatype;
	}

	/**
	 * Sets the datatype.
	 * 
	 * @param datatype
	 *            the new datatype
	 */
	public void setDatatype(String datatype) {
		this.mDatatype = datatype;
	}

	/**
	 * Gets the value.
	 * 
	 * @return the value
	 */
	public String getValue() {
		return mValue;
	}

	/**
	 * Sets the value.
	 * 
	 * @param value
	 *            the new value
	 */
	public void setValue(String value) {
		this.mValue = value;
	}

	/**
	 * Gets the minoccurs.
	 * 
	 * @return the minoccurs
	 */
	public long getMinoccurs() {
		return mMinoccurs;
	}

	/**
	 * Sets the minoccurs.
	 * 
	 * @param minoccurs
	 *            the new minoccurs
	 */
	public void setMinoccurs(String minoccurs) {
		this.mMinoccurs = convertOccurs(minoccurs);
	}

	/**
	 * Gets the maxoccurs.
	 * 
	 * @return the maxoccurs
	 */
	public long getMaxoccurs() {
		return mMaxoccurs;
	}

	/**
	 * Sets the maxoccurs.
	 * 
	 * @param maxoccurs
	 *            the new maxoccurs
	 */
	public void setMaxoccurs(String maxoccurs) {
		this.mMaxoccurs = convertOccurs(maxoccurs);
	}

	/**
	 * Sets the sub element entry list.
	 * 
	 * @param subElementEntryList
	 *            the new sub element entry list
	 */
	public void setSubElementEntryList(List<WorkItemElementEntry> subElementEntryList) {
		this.mSubElementEntryList = subElementEntryList;
	}

	/**
	 * Gets the sub element entry list.
	 * 
	 * @return the sub element entry list
	 */
	public List<WorkItemElementEntry> getSubElementEntryList() {
		return mSubElementEntryList;
	}

	/**
	 * Sets the parent.
	 * 
	 * @param parent
	 *            the new parent
	 */
	public void setParent(WorkItemElementEntry parent) {
		this.mParent = parent;
	}

	/**
	 * Gets the parent.
	 * 
	 * @return the parent
	 */
	public WorkItemElementEntry getParent() {
		return mParent;
	}

	/**
	 * Sets the level.
	 * 
	 * @param level
	 *            the new level
	 */
	public void setLevel(int level) {
		this.mLevel = level;
	}

	/**
	 * Gets the level.
	 * 
	 * @return the level
	 */
	public int getLevel() {
		return mLevel;
	}

	/**
	 * Gets the group id.
	 * 
	 * @return the group id
	 */
	public String getGroupID() {
		return mGroupID;
	}

	/**
	 * Sets the group id.
	 * 
	 * @param groupID
	 *            the new group id
	 */
	public void setGroupID(String groupID) {
		this.mGroupID = groupID;
	}

	/**
	 * Convert occurs.
	 * 
	 * @param occurs
	 *            the occurs
	 * @return the long
	 */
	private long convertOccurs(String occurs) {
		long result = 1;

		if (occurs != null) {
			if (occurs.equals("unbounded")) {
				result = Long.MAX_VALUE;
			} else {
				try {
					result = new Long(occurs);
				} catch (Exception e) {
					Log.d("WIEE", "Nothing to convert.");
				}
			}
		}
		return result;
	}

	/**
	 * Sets the occurs count.
	 * 
	 * @param occursCount
	 *            the new occurs count
	 */
	public void setOccursCount(long occursCount) {
		this.mOccursCount = occursCount;
	}

	/**
	 * Gets the occurs count.
	 * 
	 * @return the occurs count
	 */
	public long getOccursCount() {
		return mOccursCount;
	}

	/**
	 * Adds the sub field.
	 * 
	 * @param field
	 *            the field
	 */
	public void addSubField(WorkItemElementEntry field) {
		if (this.mSubElementEntryList == null) {
			this.mSubElementEntryList = new ArrayList<WorkItemElementEntry>();
		}
		field.setParent(this);
		this.mSubElementEntryList.add(field);
	}

	/**
	 * Adds the sub field list.
	 * 
	 * @param fieldList
	 *            the field list
	 */
	public void addSubFieldList(List<WorkItemElementEntry> fieldList) {
		if (fieldList != null) {
			for (WorkItemElementEntry field : fieldList) {
				addSubField(field);
			}
		}
	}

	/**
	 * Sets the order.
	 * 
	 * @param order
	 *            the new order
	 */
	public void setOrder(int order) {
		this.mOrder = order;
	}

	/**
	 * Gets the order.
	 * 
	 * @return the order
	 */
	public int getOrder() {
		return mOrder;
	}

	/**
	 * Sets the id.
	 * 
	 * @param id
	 *            the new id
	 */
	public void setId(int id) {
		this.mId = id;
	}

	/**
	 * Gets the id.
	 * 
	 * @return the id
	 */
	public int getId() {
		return mId;
	}

	/**
	 * Sets the view id.
	 * 
	 * @param viewId
	 *            the new view id
	 */
	public void setViewId(int viewId) {
		this.mViewId = viewId;
	}

	/**
	 * Gets the view id.
	 * 
	 * @return the view id
	 */
	public int getViewId() {
		return mViewId;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#clone()
	 */
	@Override
	public WorkItemElementEntry clone() {
		WorkItemElementEntry clone;
		try {
			clone = (WorkItemElementEntry) super.clone();
		} catch (CloneNotSupportedException e) {
			return null;
		}

		return clone;
	}

	/**
	 * Gets the enumerated values.
	 * 
	 * @return the enumerated values
	 */
	public List<String> getEnumeratedValues() {

		if ((mRestriction != null) && mRestriction.hasEnumeration()) {
			return mRestriction.getEnumeration();
		}

		return null;
	}

	/**
	 * Sets the enumerated values.
	 * 
	 * @param enumValues
	 *            the new enumerated values
	 */
	public void setEnumeratedValues(List<String> enumValues) {
		if (mRestriction != null) {
			mRestriction.setEnumeration(enumValues);
		}
	}

	/**
	 * Gets the restriction.
	 * 
	 * @return the restriction
	 */
	public WorkItemElementRestriction getRestriction() {
		return mRestriction;
	}

	/**
	 * Sets the restriction.
	 * 
	 * @param restriction
	 *            the new restriction
	 */
	public void setRestriction(WorkItemElementRestriction restriction) {
		this.mRestriction = restriction;
	}

	/**
	 * Checks for enumerated values.
	 * 
	 * @return true, if successful
	 */
	public boolean hasEnumeratedValues() {
		return ((mRestriction != null) && mRestriction.hasEnumeration());
	}

	/**
	 * Checks for restriction.
	 * 
	 * @return true, if successful
	 */
	public boolean hasRestriction() {
		return mRestriction != null;
	}

}
