/**
 * 
 */
package de.msponer.android.yac.xml;

import java.util.ArrayList;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Namespace;
import org.yawlfoundation.yawl.util.JDOMUtil;

import android.util.Log;

/**
 * Work item element entry deserializer. Takes the work item's XSD and XML,
 * processes the documents and generates a tree of work item element entries
 * representing the data structure. Corresponding YAWL class is
 * org.yawlfoundation.yawl.resourcing.jsf.dynform.DynFormFieldAssembler
 * 
 * @author msponer
 */
public class WorkItemElementEntryDeserializer {

	/**
	 * A unique suffix.
	 */
	private static int sUniqueSuffix;

	/**
	 * Hidden constructor. Instantiates a new work item element entry
	 * deserializer.
	 */
	protected WorkItemElementEntryDeserializer() {
	}

	/**
	 * Parses the XML and XSD data.
	 * 
	 * @param workItemDataSchemaString
	 *            the work item data schema string
	 * @param dataString
	 *            the data string
	 * @return the work item element entry
	 * @throws Exception
	 *             the exception
	 */
	public static WorkItemElementEntry parse(String workItemDataSchemaString, String dataString)
			throws Exception {

		Document doc = JDOMUtil.stringToDocument(workItemDataSchemaString);
		Element root = doc.getRootElement();
		Namespace ns = root.getNamespace();
		Element wiElement = root.getChild("element", ns);

		Element data = JDOMUtil.stringToElement(dataString);

		WorkItemElementEntry rootEntry = new WorkItemElementEntry(
				wiElement.getAttributeValue("name"),
				createElementEntryList(wiElement, data, ns, -1));

		return rootEntry;

	}

	/**
	 * Prints the element entry list.
	 * 
	 * @param wiElementEntryList
	 *            the work item element entry list
	 */
	public static void printElementEntryList(List<WorkItemElementEntry> wiElementEntryList) {

		for (WorkItemElementEntry workItemElementEntry : wiElementEntryList) {
			Log.i("WorkItemElementEntryParser",
					(workItemElementEntry.getName() + ":" + workItemElementEntry.getValue() + "("
							+ workItemElementEntry.getDatatype() + ")"));
			if (workItemElementEntry.getSubElementEntryList() != null) {
				printElementEntryList(workItemElementEntry.getSubElementEntryList());
			}
		}
	}

	/**
	 * Creates the element entry list.
	 * 
	 * @param wiElement
	 *            the wi element
	 * @param data
	 *            the data
	 * @param ns
	 *            the name space
	 * @param level
	 *            the level
	 * @return the list
	 * @throws Exception
	 *             the exception
	 */
	private static List<WorkItemElementEntry> createElementEntryList(Element wiElement,
			Element data, Namespace ns, int level) throws Exception {
		List<WorkItemElementEntry> elementEntryList = null;

		++level;

		if (wiElement.getName().equals("choice")) {
			throw new Exception("'choice' is not yet supported.");
		} else {
			Element complex = wiElement.getChild("complexType", ns);
			if (complex == null) {
				throw new Exception("Schema invalid at element: "
						+ JDOMUtil.elementToString(wiElement));
			}

			Element sequence = complex.getChild("sequence", ns);

			if (sequence == null) {
				sequence = complex.getChild("all", ns);
			}

			if (sequence != null) {
				elementEntryList = createSequence(sequence, data, ns, level);
			} else {
				sequence = complex.getChild("choice", ns);
				if (sequence != null) {
					throw new Exception("'choice' not yet supported.");
				}
			}
		}

		return elementEntryList;
	}

	/**
	 * Creates an element sequence.
	 * 
	 * @param sequence
	 *            the sequence
	 * @param data
	 *            the data
	 * @param ns
	 *            the name space
	 * @param level
	 *            the level
	 * @return the list
	 * @throws Exception
	 *             the exception
	 */
	private static List<WorkItemElementEntry> createSequence(Element sequence, Element data,
			Namespace ns, int level) throws Exception {

		List<WorkItemElementEntry> fieldList = new ArrayList<WorkItemElementEntry>();

		for (Object o : sequence.getChildren()) {
			Element eField = (Element) o;
			List<WorkItemElementEntry> result = createElementEntry(eField, data, ns, level);
			setOrderForListItems(result, fieldList.size());
			fieldList.addAll(result);
		}
		return fieldList;
	}

	/**
	 * Creates the element entry.
	 * 
	 * @param eField
	 *            the e field
	 * @param data
	 *            the data
	 * @param ns
	 *            the name space
	 * @param level
	 *            the level
	 * @return the list
	 * @throws Exception
	 *             the exception
	 */
	@SuppressWarnings("rawtypes")
	private static List<WorkItemElementEntry> createElementEntry(Element eField, Element data,
			Namespace ns, int level) throws Exception {

		WorkItemElementEntry field = null;
		List<WorkItemElementEntry> result = new ArrayList<WorkItemElementEntry>();

		String minOccurs = eField.getAttributeValue("minOccurs");
		String maxOccurs = eField.getAttributeValue("maxOccurs");
		String name = eField.getAttributeValue("name");
		String type = eField.getAttributeValue("type");

		if (type == null) {

			// check for a simpleType definition
			Element simple = eField.getChild("simpleType", ns);
			if (simple != null) {
				field = addField(name, null, data, minOccurs, maxOccurs, level);
				applySimpleType(simple, ns, field);
				result.add(field);
			} else {
				// new complex type - recurse in a new field list
				String groupID = getNextGroupID();

				List dataList = null;
				if (data != null) {
					dataList = data.getChildren(name);
				}

				if ((dataList != null) && (!dataList.isEmpty())) {
					for (Object o : dataList) {
						field = addGroupField(name, eField, ns, (Element) o, minOccurs, maxOccurs,
								level);
						field.setGroupID(groupID);
						result.add(field);
					}
				} else {
					field = addGroupField(name, eField, ns, null, minOccurs, maxOccurs, level);
					field.setGroupID(groupID);
					result.add(field);
				}
			}
		} else {
			// a plain element
			result.addAll(addElementField(name, type, data, minOccurs, maxOccurs, level));
		}

		return result;
	}

	/**
	 * Adds the attribute field.
	 * 
	 * @param name
	 *            the name
	 * @param type
	 *            the type
	 * @param data
	 *            the data
	 * @param minOccurs
	 *            the min occurs
	 * @param maxOccurs
	 *            the max occurs
	 * @param level
	 *            the level
	 * @return the work item element entry
	 */
	private static WorkItemElementEntry addField(String name, String type, Element data,
			String minOccurs, String maxOccurs, int level) {

		String value = "";
		if (data != null) {
			value = data.getChildText(name);
		}
		WorkItemElementEntry input = new WorkItemElementEntry(name, type, value);
		populateField(input, name, data, minOccurs, maxOccurs, level);
		return input;
	}

	/**
	 * Adds the attribute field.
	 * 
	 * @param name
	 *            the name
	 * @param subFieldList
	 *            the sub field list
	 * @param minOccurs
	 *            the min occurs
	 * @param maxOccurs
	 *            the max occurs
	 * @param level
	 *            the level
	 * @return the work item element entry
	 */
	private static WorkItemElementEntry addField(String name,
			List<WorkItemElementEntry> subFieldList, String minOccurs, String maxOccurs, 
			int level) {
		if (name == null) {
			name = "choice";
		}
		WorkItemElementEntry input = new WorkItemElementEntry(name, subFieldList);
		populateField(input, name, null, minOccurs, maxOccurs, level);
		return input;
	}

	/**
	 * Populates the field.
	 * 
	 * @param input
	 *            the input
	 * @param name
	 *            the name
	 * @param data
	 *            the data
	 * @param minOccurs
	 *            the min occurs
	 * @param maxOccurs
	 *            the max occurs
	 * @param level
	 *            the level
	 */
	private static void populateField(WorkItemElementEntry input, String name, Element data,
			String minOccurs, String maxOccurs, int level) {
		input.setMinoccurs(minOccurs);
		input.setMaxoccurs(maxOccurs);
		input.setLevel(level);
	}

	/**
	 * Returns the next group id.
	 * 
	 * @return the next group id
	 */
	private static String getNextGroupID() {
		return "group" + String.valueOf(sUniqueSuffix++);
	}

	/**
	 * Adds the element attribute field.
	 * 
	 * @param name
	 *            the name
	 * @param type
	 *            the type
	 * @param data
	 *            the data
	 * @param minOccurs
	 *            the min occurs
	 * @param maxOccurs
	 *            the max occurs
	 * @param level
	 *            the level
	 * @return the list
	 */
	private static List<WorkItemElementEntry> addElementField(String name, String type,
			Element data, String minOccurs, String maxOccurs, int level) {

		List<WorkItemElementEntry> result = new ArrayList<WorkItemElementEntry>();
		WorkItemElementEntry field;
		String groupID = null;
		boolean cloneable = isCloneableField(minOccurs, maxOccurs);

		if (cloneable) {
			groupID = getNextGroupID();
		}

		int instances = getInitialInstanceCount(minOccurs, data, name);

		for (int i = 0; i < instances; i++) {
			Element data4Inst;
			if (instances > 1) {
				data4Inst = getIteratedContent(data, i, name);
			} else {
				data4Inst = data;
			}
			field = addField(name, type, data4Inst, minOccurs, maxOccurs, level);
			field.setGroupID(groupID);
			field.setOccursCount(instances);
			result.add(field);
		}
		return result;
	}

	/**
	 * Checks if the field is cloneable.
	 * 
	 * @param minoccurs
	 *            the minoccurs
	 * @param maxoccurs
	 *            the maxoccurs
	 * @return true, if field is cloneable
	 */
	private static boolean isCloneableField(String minoccurs, String maxoccurs) {

		int min = convertOccurs(minoccurs);
		int max = convertOccurs(maxoccurs);
		return (max > 1) && (max > min);
	}

	/**
	 * Returns the initial instance count.
	 * 
	 * @param min
	 *            the min
	 * @param data
	 *            the data
	 * @param dataName
	 *            the data name
	 * @return the initial instance count
	 */
	private static int getInitialInstanceCount(String min, Element data, String dataName) {
		int dataCount = 1;
		int minOccurs = Math.max(convertOccurs(min), 1);
		if ((data != null) && (data.getContentSize() > 1)) {
			dataCount = data.getChildren(dataName).size();
		}
		return Math.max(minOccurs, dataCount);
	}

	/**
	 * Returns the iterated content.
	 * 
	 * @param data
	 *            the data
	 * @param index
	 *            the index
	 * @param name
	 *            the name
	 * @return the iterated content
	 */
	@SuppressWarnings("rawtypes")
	private static Element getIteratedContent(Element data, int index, String name) {
		Element result = null;
		if ((data != null) && (index < data.getContentSize())) {
			List relevantChildren = data.getChildren(name);
			result = new Element(data.getName());
			Element iteratedContent = (Element) relevantChildren.get(index);
			result.addContent((Element) iteratedContent.clone());
		}
		return result;
	}

	/**
	 * Convert occurs.
	 * 
	 * @param occurs
	 *            the occurs
	 * @return the int
	 */
	private static int convertOccurs(String occurs) {
		int result = 1;

		if (occurs != null) {
			if (occurs.equals("unbounded")) {
				result = Integer.MAX_VALUE;
			} else {
				try {
					result = new Integer(occurs);
				} catch (Exception e) {
					Log.e("WIEEDS", "Exception: " + e.getMessage());
				}
			}
		}
		return result;
	}

	/**
	 * Adds the group field.
	 * 
	 * @param name
	 *            the name
	 * @param eField
	 *            the e field
	 * @param ns
	 *            the ns
	 * @param data
	 *            the data
	 * @param minOccurs
	 *            the min occurs
	 * @param maxOccurs
	 *            the max occurs
	 * @param level
	 *            the level
	 * @return the work item element entry
	 * @throws Exception
	 *             the exception
	 */
	private static WorkItemElementEntry addGroupField(String name, Element eField, Namespace ns,
			Element data, String minOccurs, String maxOccurs, int level) throws Exception {

		WorkItemElementEntry field;
		List<WorkItemElementEntry> simpleContents = new ArrayList<WorkItemElementEntry>();
		int instances = getInitialInstanceCount(minOccurs, data, name);

		if (instances == 1) {
			field = addField(name, createElementEntryList(eField, data, ns, level), minOccurs,
					maxOccurs, level);
		} else {
			field = addContainingField(name, minOccurs, maxOccurs, level);
			String subGroupID = getNextGroupID();
			for (int i = 0; i < instances; i++) {
				Element data4Inst = getIteratedContent(data, i, name);
				List<WorkItemElementEntry> subFieldList = createElementEntryList(eField, data4Inst,
						ns, level);
				WorkItemElementEntry subField = subFieldList.get(0);

				// field
				subField.setGroupID(subGroupID);
				field.addSubField(subField);
				simpleContents = addSimpleContents(simpleContents, subFieldList);
			}
		}
		field.setOccursCount(instances);
		field.addSubFieldList(simpleContents);
		return field;
	}

	/**
	 * Adds the containing field.
	 * 
	 * @param name
	 *            the name
	 * @param minOccurs
	 *            the min occurs
	 * @param maxOccurs
	 *            the max occurs
	 * @param level
	 *            the level
	 * @return the work item element entry
	 */
	private static WorkItemElementEntry addContainingField(String name, String minOccurs,
			String maxOccurs, int level) {
		if (name == null) {
			name = "choice";
		}
		WorkItemElementEntry input = new WorkItemElementEntry(name, null);
		input.setMinoccurs(minOccurs);
		input.setMaxoccurs(maxOccurs);
		input.setLevel(level);
		return input;
	}

	/**
	 * Adds simple contents.
	 * 
	 * @param simpleContents
	 *            the simple contents
	 * @param subFieldList
	 *            the sub field list
	 * @return the list
	 */
	private static List<WorkItemElementEntry> addSimpleContents(
			List<WorkItemElementEntry> simpleContents, List<WorkItemElementEntry> subFieldList) {

		if (subFieldList == null) {
			return simpleContents;
		}

		boolean contains = false;

		for (int i = 1; i < subFieldList.size(); i++) {
			WorkItemElementEntry field = subFieldList.get(i);
			for (WorkItemElementEntry simple : simpleContents) {
				contains = simple.equals(field);
				if (contains) {
					break;
				}
			}
			if (!contains) {
				simpleContents.add(field);
			}
		}
		return simpleContents;
	}

	/**
	 * Sets the order for list items.
	 * 
	 * @param list
	 *            the list
	 * @param order
	 *            the order
	 */
	private static void setOrderForListItems(List<WorkItemElementEntry> list, int order) {
		for (WorkItemElementEntry field : list) {
			field.setOrder(order++);
		}
	}

	/**
	 * Applies a simple type.
	 * 
	 * @param simple
	 *            the simple element
	 * @param ns
	 *            the name space
	 * @param field
	 *            the field
	 */
	private static void applySimpleType(Element simple, Namespace ns, WorkItemElementEntry field) {

		Element restriction = simple.getChild("restriction", ns);
		if (restriction != null) {
			WorkItemElementRestriction restrict = new WorkItemElementRestriction(restriction, ns);
			field.setDatatype(restrict.getBaseType());
			field.setRestriction(restrict);
		}
	}
}
