package de.msponer.android.yac.xml;

import java.util.ArrayList;
import java.util.List;

import org.jdom.Element;
import org.jdom.Namespace;

/**
 * Class represents a XSD restriction element. YAC uses it to support
 * enumeration. Other attributes are not yet supported. The corresponding YAWL
 * class is
 * org.yawlfoundation.yawl.resourcing.jsf.dynform.DynFormFieldRestriction
 * 
 * @author msponer
 */
public class WorkItemElementRestriction {
	/**
	 * The work item element entry that owns the current restriction.
	 */
	private WorkItemElementEntry mOwner;

	/**
	 * The base type.
	 */
	private String mBaseType;

	/**
	 * The list of enumerations.
	 */
	private List<String> mEnumeration;

	/**
	 * Instantiates a new work item element restriction.
	 * 
	 * @param restriction
	 *            the restriction
	 * @param ns
	 *            the ns
	 */
	public WorkItemElementRestriction(Element restriction, Namespace ns) {

		parse(restriction, ns);
	}

	/**
	 * Parses the restriction.
	 * 
	 * @param restriction
	 *            the restriction
	 * @param ns
	 *            the name space
	 */
	private void parse(Element restriction, Namespace ns) {
		mBaseType = restriction.getAttributeValue("base");
		if (mBaseType == null) {
			mBaseType = parseSimpleTypeBase(restriction, ns);
		}
		mEnumeration = getEnumeratedValues(restriction, ns);
	}

	/**
	 * Returns the enumerated values.
	 * 
	 * @param restriction
	 *            the restriction
	 * @param ns
	 *            the name space
	 * @return the enumerated values
	 */
	@SuppressWarnings("rawtypes")
	private List<String> getEnumeratedValues(Element restriction, Namespace ns) {
		List<String> result = null;
		if (restriction != null) {
			List enumChildren = restriction.getChildren("enumeration", ns);
			if ((enumChildren != null) && !enumChildren.isEmpty()) {
				result = new ArrayList<String>();
				for (int i = 0; i < enumChildren.size(); i++) {
					Element enumChild = (Element) enumChildren.get(i);
					result.add(enumChild.getAttributeValue("value"));
				}
			}
		}
		return result;
	}

	/**
	 * Parses the simple type base.
	 * 
	 * @param restriction
	 *            the restriction
	 * @param ns
	 *            the name space
	 * @return the string
	 */
	@SuppressWarnings("rawtypes")
	private String parseSimpleTypeBase(Element restriction, Namespace ns) {
		if (restriction != null) {
			Element simpleType = restriction.getChild("simpleType", ns);
			if (simpleType != null) {
				List children = simpleType.getChildren();
				Element content = (Element) children.get(0);
				return content.getAttributeValue("itemType");
			}
		}
		return null;
	}

	/**
	 * Returns the base type.
	 * 
	 * @return the base type
	 */
	public String getBaseType() {
		return mBaseType;
	}

	/**
	 * Sets the base type.
	 * 
	 * @param baseType
	 *            the new base type
	 */
	public void setBaseType(String baseType) {
		this.mBaseType = baseType;
	}

	/**
	 * Returns the owner.
	 * 
	 * @return the owner
	 */
	public WorkItemElementEntry getOwner() {
		return mOwner;
	}

	/**
	 * Sets the owner.
	 * 
	 * @param owner
	 *            the new owner
	 */
	public void setOwner(WorkItemElementEntry owner) {
		this.mOwner = owner;
	}

	/**
	 * Returns the enumeration.
	 * 
	 * @return the enumeration
	 */
	public List<String> getEnumeration() {
		return mEnumeration;
	}

	/**
	 * Sets the enumeration.
	 * 
	 * @param enumeration
	 *            the new enumeration
	 */
	public void setEnumeration(List<String> enumeration) {
		this.mEnumeration = enumeration;
	}

	/**
	 * Checks for enumeration.
	 * 
	 * @return true, if successful
	 */
	public boolean hasEnumeration() {
		return mEnumeration != null;
	}

}
